/**
 * EPUBReader Firefox Extension: http://www.epubread.com/
 * Copyright (C) 2009 Michael Volz (epubread@gmail.com)
 *
 * This program is free software: you can redistribute it under
 * the terms of the attached license (license.txt).
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * attached license (license.txt) for more details.
 *
 * You should have received a copy of the license (license.txt)
 * along with this program. If not, see <http://www.epubread.com/license/>.
**/

const Ci = Components.interfaces;
const Cc = Components.classes;

var mylog =
{
   file: null,
   stack: null,

   start: function()
   {
      this.stack = "";

      var start = new Date();
      this.log("------------------\nstart: " +  start.toUTCString());
   },

   getStack: function()
   {
      return this.stack;
   },

   enabled: function()
   {
      var pref = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("extensions.epubreader.");
      var logging = pref.getBoolPref("logging");

      return logging;
   },

   log: function(data)
   {
      if(this.enabled())
      {
         if(!file)
         {
            var dirLocator = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
            var file = dirLocator.get("ProfD", Ci.nsIFile);
            file.append("epub");
            file.append("epub.log");
            this.file = file;
         }

         epub.writeFile(file, data + "\n", true, true);
         dump(data + "\n");
      }

      this.stack += data + "<br>";
   }
}

/* epub-class, here is all the work done */
var epub =
{
   app: null,
   file: null,
   fileDir: null,
   docRoot: null,
   epubRoot: null,
   isInitialized: false,
   catalog: null,

   exception_encrypted: -1,
   exception_corrupted: -2,
   exception_too_large: -3,
   exception_invalid_characters: -4,
   exception_container_missing: -5,
   exception_opf_missing: -6,
   exception_http_status_not_found: -7,
   exception_http_status_not_authorized: -8,
   exception_http_status_other: -9,
   exception_other: -10,

   app_firefox: "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
   app_seamonkey: "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}",
   app_fennec: "{a23983c0-fd0e-11dc-95ff-0800200c9a66}",

   init: function(upgrade)
   {
      if(!this.isInitialized)
      {
         try
         {
            var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
            this.app = appInfo.ID;

            /* set epubRoot */
            var dirLocator = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
            var userPrevPath = dirLocator.get("ProfD", Ci.nsIFile);

            userPrevPath.append("epub");
            this.epubRoot = userPrevPath.clone();

            /* create epub-root if it not exists*/
            if(!this.epubRoot.exists())
            {
               this.epubRoot.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
            }

            /* copy css */
            this.copyCSS();

            /* init database */
            this.sqlite.init(this.epubRoot, upgrade);

            /* cleanup epub directory */
            if(upgrade == false)
            {
               this.cleanUp();
            }

            this.isInitialized = true;
         }
         catch(e)
         {
            mylog.log(e);
            throw(e);
         }
      }
   },

   cleanUp: function()
   {
      function removeRecursive(dir)
      {
         var entries = dir.directoryEntries;

         while(entries.hasMoreElements())
         {
            var entry = entries.getNext();
            entry.QueryInterface(Components.interfaces.nsIFile);

            if(entry.exists())
            {
               if(entry.isDirectory())
               {
                  removeRecursive(entry);
               }
               else
               {
                  try
                  {
                     entry.permissions = 0777;
                     entry.remove(false);
                  }
                  catch(e)
                  {
                     dump(e + "\n");
                  }
               }
            }
         }
      }

      var dirLocator = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
      var userPrevPath = dirLocator.get("ProfD", Ci.nsIFile);
      var regExp = new RegExp(userPrevPath + ".*");

      var entries = this.epubRoot.directoryEntries;

      while(entries.hasMoreElements())
      {
         var entry = entries.getNext();
         entry.QueryInterface(Components.interfaces.nsIFile);

         if(entry.exists())
         {
            if(entry.isDirectory())
            {
               var epub = this.sqlite.getEpubByPath(entry.leafName);

               if(!epub)
               {
                  if(entry.path.match(regExp))
                  {
                     dump("cleanUp: " + entry.path + "\n");

                     try
                     {
                        removeRecursive(entry);
                        entry.remove(true);
                     }
                     catch(e)
                     {
                        dump(e + "\n");
                     }
                  }
               }
            }
         }
      }
   },

   copyCSS: function()
   {
      var html =
      {
         get: function(epubRoot, template)
         {
            var paging_dir_horizontal = 1;
            var paging_dir_vertical = 2;

            var pref = Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefService).getBranch("extensions.epubreader.");

            var root = "file:///" + encodeURI(epubRoot.path.replace(/\\/g, "/"));
            var contentMargin = pref.getCharPref("margin.content");
            var contentPadding = pref.getCharPref("padding.content");
            var contentBgColor = pref.getCharPref("color.content.background");
            var contentFontSize = pref.getCharPref("font.size.content");
            var contentFontColor = pref.getCharPref("font.color.content");
            var contentFontFamily = pref.getCharPref("font.family.content");
            var contentColumnCount = pref.getCharPref("count.column.content");
            var contentColumnWidth = pref.getCharPref("width.column.content");
            var contentPagingDir = pref.getIntPref("paging_dir.content");
            var contentLineHeight = pref.getCharPref("line_height.content");
            var navLeftBgColor = pref.getCharPref("color.nav.background");
            var navLeftFontFamily = pref.getCharPref("font.nav.link");
            var navLeftLinkUnvisitedColor = pref.getCharPref("color.nav.link.unvisited");
            var navLeftLinkVisitedColor = pref.getCharPref("color.nav.link.visited");
            var navLeftLinkHoverColor = pref.getCharPref("color.nav.link.hover");
            var navLeftLinkFontSize = pref.getCharPref("size.nav.link");

            template = template.replace(/%root%/g, root);

            if(contentMargin != "parent")
            {
               template = template.replace(/%contentMargin%/g, contentMargin);
               template = template.replace(/%contentMarginBegin%/g, "");
               template = template.replace(/%contentMarginEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentMarginBegin%/g, "/*");
               template = template.replace(/%contentMarginEnd%/g, "*/");
            }

            if(contentPadding != "parent")
            {
               template = template.replace(/%contentPadding%/g, contentPadding);
               template = template.replace(/%contentPaddingBegin%/g, "");
               template = template.replace(/%contentPaddingEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentPaddingBegin%/g, "/*");
               template = template.replace(/%contentPaddingEnd%/g, "*/");
            }

            if(contentBgColor != "parent")
            {
               template = template.replace(/%contentBackgroundColor%/g, contentBgColor);
               template = template.replace(/%contentBackgroundColorBegin%/g, "");
               template = template.replace(/%contentBackgroundColorEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentBackgroundColorBegin%/g, "/*");
               template = template.replace(/%contentBackgroundColorEnd%/g, "*/");
            }

            if(contentFontSize != "parent")
            {
               template = template.replace(/%contentFontSize%/g, contentFontSize);
               template = template.replace(/%contentFontSizeBegin%/g, "");
               template = template.replace(/%contentFontSizeEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentFontSizeBegin%/g, "/*");
               template = template.replace(/%contentFontSizeEnd%/g, "*/");
            }

            if(contentFontColor != "parent")
            {
               template = template.replace(/%contentFontColor%/g, contentFontColor);
               template = template.replace(/%contentFontColorBegin%/g, "");
               template = template.replace(/%contentFontColorEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentFontColorBegin%/g, "/*");
               template = template.replace(/%contentFontColorEnd%/g, "*/");
            }

            if(contentFontFamily != "parent")
            {
               template = template.replace(/%contentFontFamily%/g, contentFontFamily);
               template = template.replace(/%contentFontFamilyBegin%/g, "");
               template = template.replace(/%contentFontFamilyEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentFontFamilyBegin%/g, "/*");
               template = template.replace(/%contentFontFamilyEnd%/g, "*/");
            }

            if(contentColumnCount != "single")
            {
               template = template.replace(/%contentColumnWidth%/g, contentColumnWidth);
               template = template.replace(/%contentColumnCount%/g, contentColumnCount);
               template = template.replace(/%contentColumnBegin%/g, "");
               template = template.replace(/%contentColumnEnd%/g, "");
            }
            else
            {
               template = template.replace(/%contentColumnCount%/g, "1");
               template = template.replace(/%contentColumnBegin%/g, "/*");
               template = template.replace(/%contentColumnEnd%/g, "*/");
            }

            if(contentPagingDir == paging_dir_horizontal)
            {
               template = template.replace(/%contentPagingDirBegin%/g, "/*");
               template = template.replace(/%contentPagingDirEnd%/g, "*/");
               template = template.replace(/%contentColumnFill%/g, "auto");
            }
            else
            {
               template = template.replace(/%contentPagingDirBegin%/g, "");
               template = template.replace(/%contentPagingDirEnd%/g, "");
               template = template.replace(/%contentColumnFill%/g, "balance");
            }

            template = template.replace(/%contentLineHeight%/g, contentLineHeight);
            template = template.replace(/%navLeftBackgroundColor%/g, navLeftBgColor);
            template = template.replace(/%navLeftFontFamily%/g, navLeftFontFamily);
            template = template.replace(/%navLeftLinkUnvisitedColor%/g, navLeftLinkUnvisitedColor);
            template = template.replace(/%navLeftLinkVisitedColor%/g, navLeftLinkVisitedColor);
            template = template.replace(/%navLeftLinkHoverColor%/g, navLeftLinkHoverColor);
            template = template.replace(/%navLeftLinkFontSize%/g, navLeftLinkFontSize);

            return template;
         }
      }

      mylog.log("copyCSS: begin");

      var template = this.readURL("chrome://epubreader/content/epub_template.css");
      var content = html.get(this.epubRoot, template);

      file = this.epubRoot.clone();
      file.append("epub.css");
      this.writeFile(file, content, false, true);

      mylog.log("copyCSS: end");
   },

   sqlite:
   {
      db : null,
      file : null,

      init: function(file, upgrade)
      {
         this.file = file.clone();
         this.connect();
         this.createEpubTables();

         if(upgrade == false)
         {
            this.fillEpubTables();
         }
      },

      connect: function()
      {
         this.file.append("epub.sqlite");

         var storageService = Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);
         var db = storageService.openDatabase(this.file);
         this.db = db;
      },

      executeSimple: function(sql)
      {
         //mylog.log("executeSimple: " + sql);

         try
         {
            this.db.executeSimpleSQL(sql);
         }
         catch(e)
         {
            mylog.log("ExecuteSimple: " + sql + ", " + this.db.lastErrorString);
            throw(e);
         }
      },

      select: function(sql)
      {
         //mylog.log("select: " + sql);

         try
         {
	        var statement = this.db.createStatement(sql);
	     }
	     catch(e)
	     {
            mylog.log("Select: " + sql + ", " + this.db.lastErrorString);
            throw(e);
	     }

		 try
		 {
	        var dataset = [];

			while(statement.executeStep())
			{
			   var row = [];

			   for(var i = 0, k = statement.columnCount; i < k; i++)
			   {
			      row[statement.getColumnName(i)] = statement.getUTF8String(i);
			   }

               dataset.push(row);
			}
		 }
		 finally
		 {
	        statement.reset();
		 }

		 return dataset;
      },

      createEpubTables: function()
      {
         var sql = "create table if not exists book(id integer primary key autoincrement, identifier text, metadata integer, path text, name text, title text, author text, url text, first text, status integer, timestamp integer)";
         this.executeSimple(sql);

         var sql = "create table if not exists metadata(id integer primary key autoincrement, book_id integer, type integer, subtype text, data text)";
         this.executeSimple(sql);

         var sql = "create table if not exists id(id integer)";
         this.executeSimple(sql);

         var sql = "create table if not exists bookmark(id integer primary key autoincrement, book_id integer, name text, page text, text text)";
         this.executeSimple(sql);

         var sql = "create table if not exists catalog(id integer primary key autoincrement, name text, start text, search text, language text, code text)";
         this.executeSimple(sql);

         var sql = "create table if not exists tag(id integer primary key autoincrement, name text, book_count integer)";
         this.executeSimple(sql);

         var sql = "create table if not exists book_2_tag(book_id integer, tag_id integer)";
         this.executeSimple(sql);
      },

      fillEpubTables: function()
      {
         var sql = "select count(*) from id";
         var data = this.select(sql);

         if(data[0]['count(*)'] == 0)
         {
            var sql = "insert into id(id) values(0)";
            this.executeSimple(sql);
         }

         var catalog = [];
         catalog['name'] = "Archive.org";
         catalog['start'] = "http://bookserver.archive.org/catalog/";
         catalog['search'] = "http://bookserver.archive.org/catalog/opensearch?q=%search%";
         catalog['language'] = "";
         catalog['code'] = "8bf5dbdb4e585d87b4d81650b692e082";
         this.storeCatalog(catalog);

         var catalog = [];
         catalog['name'] = "Feedbooks.com";
         catalog['start'] = "http://www.feedbooks.com/site/free_books.atom";
         catalog['search'] = "";
         catalog['language'] = "";
         catalog['code'] = "d498aeb382c67f941bdefbe7114b6120";
         this.storeCatalog(catalog);

         var catalog = [];
         catalog['name'] = "EPUBReaders eBook Shop";
         catalog['start'] = "http://www.epubbuy.com/opds.php";
         catalog['search'] = "http://www.epubbuy.com/opds.php?type=items&from=search&query=%search%";
         catalog['language'] = "";
         catalog['code'] = "ef57c8cbcb29d1453265e807bf91afdf";
         this.storeCatalog(catalog);

         var catalog = [];
         catalog['name'] = "Gutenberg.org";
         catalog['start'] = "http://m.gutenberg.org/ebooks/?format=mobile";
         catalog['search'] = "";
         catalog['language'] = "";
         catalog['code'] = "0f7f6f2724fad54ad23e2d0e6e380aab";
         this.storeCatalog(catalog);
      },

      getEpubId: function()
      {
         var sql = "update id set id = id + 1";
         this.executeSimple(sql);

         var sql = "select id from id";
         var data = this.select(sql);

         var id = data[0]['id'];

         return id;
      },

      storeEpub: function(epub)
      {
         var sql = 'select id from book where identifier = "' + epub['identifier'] + '" and identifier <> "-"';
         var data = this.select(sql);
         mylog.log("storeEpub: " + sql);

         if(data[0])
         {
            var sql = 'update book set path = "' + epub['path'] + '", name = "' + epub['name'] + '", title = "' + epub['title'].replace(/['"]/g, "") + '", author = "' + epub['author'].replace(/['"]/g, "") + '", url = "' + epub['url'] + '", first = "' + epub['first'] + '", status = 1, timestamp = strftime("%s","now") where id = ' + data[0]['id'];
            this.executeSimple(sql);
            mylog.log("storeEpub: " + sql);

            var id = data[0]['id'];
         }
         else
         {
            var sql = 'insert into book(identifier, path, name, title, author, url, first, status, timestamp) values("' + epub['identifier'] + '", "' + epub['path'] + '", "' + epub['name'] + '", "' + epub['title'].replace(/['"]/g, "") + '", "' + epub['author'].replace(/['"]/g, "") + '", "' + epub['url'] + '", "' + epub['first'] + '", 1, strftime("%s","now"))';
            this.executeSimple(sql);
            mylog.log("storeEpub: " + sql);

            var sql = 'select max(id) from book';
            var data = this.select(sql);
            var id = data[0]['max(id)'];
         }

         return id;
      },

      deleteEpub: function(id)
      {
         var sql = "select path from book where id = " + id;
         var data = this.select(sql);
         var path = data[0]['path'];

         var sql = 'update book set status = 0, timestamp = strftime("%s","now") where id = ' + id;
         this.executeSimple(sql);

         this.deleteBookTags(id);

         return path;
      },

      deleteEpubPermanent: function(id)
      {
         var sql = 'delete from book where id = ' + id;
         this.executeSimple(sql);

         this.deleteBookTags(id);
      },

      deleteEpubsPermanent: function()
      {
         var sql = "delete from book where status = 0";
         this.executeSimple(sql);
      },

      getEpubs: function(tag_id)
      {
         if(!tag_id)
         {
            var sql = "select * from book where book.status = 1";
         }
         else
         {
            var sql = "select * from book, book_2_tag where book.status = 1 and book.id = book_2_tag.book_id and book_2_tag.tag_id = " + tag_id;
         }

         var epub = this.select(sql);

         return epub;
      },

      getDeletedEpubs: function()
      {
         var sql = "select * from book where status = 0";
         var epub = this.select(sql);

         return epub;
      },

      getEpub: function(id)
      {
         var sql = "select * from book where id = " + id + " and status = 1";
         var epub = this.select(sql);

         return epub[0];
      },

      getEpubByPath: function(path)
      {
         var sql = 'select * from book where path = "' + path + '" and status = 1';
         var epub = this.select(sql);

         return epub[0];
      },

      getEpubByIdentifier: function(id)
      {
         var sql = 'select * from book where identifier = "' + id + '" and status = 1';
         var epub = this.select(sql);

         return epub[0];
      },

      storeMetadata: function(metadata)
      {
         var sql = 'insert into metadata(book_id, type, subtype, data) values(' + metadata['id'] + ', ' + metadata['type'] + ', "' + metadata['subtype'] + '", "' + metadata['data'] + '")';
         mylog.log("storeMetadata: " + sql);
         this.executeSimple(sql);
      },

      storeBookMetadata: function(id, count)
      {
         var sql = 'update book set metadata = ' + count + ' where id = ' + id;
         mylog.log("storeBookMetadata: " + sql);
         this.executeSimple(sql);
      },

      deleteMetadata: function(id)
      {
         var sql = 'delete from metadata where book_id = ' + id;
         mylog.log("deleteAllMetadata: " + sql);
         this.executeSimple(sql);
      },

      getMetadata: function(id)
      {
         var sql = 'select * from metadata where book_id = ' + id + ' order by type';
         var meta = this.select(sql);

         return meta;
      },

      storeBookmark: function(bookmark)
      {
         var sql = 'select count(*) from bookmark where book_id = ' + bookmark['id'] + ' and name = "' + bookmark['name'] + '"';
         var data = this.select(sql);

         if(data[0]['count(*)'] == 0)
         {
            var sql = "insert into bookmark(book_id, name, page, text) values(" + bookmark['id'] + ", '" + bookmark['name'] + "', '" + bookmark['page'] + "', '" + bookmark['text'] + "')";
         }
         else
         {
            var sql = "update bookmark set page = '" + bookmark['page'] + "', text = '" + bookmark['text'] + "' where book_id = " + bookmark['id'] + " and name = '" + bookmark['name'] + "'";
         }

         mylog.log("storeBookmark: " + sql);
         this.executeSimple(sql);
      },

      getBookmarks: function(id)
      {
         var sql = "select * from bookmark where book_id = " + id;
         var bookmarks = this.select(sql);

         return bookmarks;
      },

      getBookmarkByName: function(id, name)
      {
         var sql = 'select * from bookmark where book_id = ' + id + ' and name = "' + name + '"';
         mylog.log("getStandardBookmark: " + sql);
         var bookmark = this.select(sql);

         return bookmark[0];
      },

      storeCatalog: function(catalog)
      {
         var sql = 'select count(*) from catalog where name = "' + catalog['name'] + '"';
         var data = this.select(sql);

         if(data[0]['count(*)'] == 0)
         {
            var sql = 'insert into catalog(name, start, search, language, code) values("' + catalog['name'] + '", "' + catalog['start'] + '", "' + catalog['search'] + '", "' + catalog['language'] + '", "' + catalog['code'] + '")';
         }
         else
         {
            var sql = 'update catalog set start = "' + catalog['start'] + '", search = "' + catalog['search'] + '", language = "' + catalog['language'] + '", code = "' + catalog['code'] + '" where name = "' + catalog['name'] + '"';
         }

         this.executeSimple(sql);
      },

      getCatalogs: function()
      {
         var sql = "select * from catalog where start <> '' order by name";
         var catalogs = this.select(sql);

         return catalogs;
      },

      getCatalogByName: function(name)
      {
         var sql = 'select * from catalog where name = "' + name + '"';
         var catalog = this.select(sql);

         return catalog[0];
      },

      storeTag: function(name)
      {
         var sql = 'select count(*) from tag where upper(name) = "' + name.toUpperCase() + '"';
         var data = this.select(sql);

         if(data[0]['count(*)'] == 0)
         {
            var sql = "insert into tag(name, book_count) values('" + name + "', 0)";
            this.executeSimple(sql);
            return true;
         }
         else
         {
            return -1;
         }
      },

      updateTag: function(id, name)
      {
         var sql = 'select id from tag where upper(name) = "' + name.toUpperCase() + '"';
         var data = this.select(sql);

         if(data.length == 0 || (data.length > 0 && data[0]['id'] == id))
         {
            var sql = "update tag set name = '" + name + "' where id = " + id;
            this.executeSimple(sql);
            return true;
         }
         else
         {
            return -1;
         }
      },

      deleteTag: function(id)
      {
         var sql = 'delete from book_2_tag where tag_id = ' + id;
         this.executeSimple(sql);

         var sql = 'delete from tag where id = ' + id;
         this.executeSimple(sql);
      },

      getTags: function()
      {
         var sql = 'select * from tag order by upper(name) asc';
         var tags = this.select(sql);

         return tags;
      },

      getBookTags: function(id)
      {
         var sql = 'select * from book_2_tag, tag where book_2_tag.book_id = ' + id + ' and book_2_tag.tag_id = tag.id';
         var tags = this.select(sql);

         return tags;
      },

      storeBookTags: function(id, bookTags)
      {
         var sql = 'select tag_id from book_2_tag where book_id = ' + id;
         var tags = this.select(sql);

         for(var i = 0; i < tags.length; i++)
         {
            var found = false;

            for(var j = 0; j < bookTags.length; j++)
            {
               if(bookTags[j] == tags[i].id)
               {
                  found = true;
                  break;
               }
            }

            if(found == false)
            {
               var sql = 'delete from book_2_tag where book_id = ' + id + ' and tag_id = ' + tags[i].tag_id;
               this.executeSimple(sql);

               var sql = 'update tag set book_count = book_count - 1 where id = ' + tags[i].tag_id;
               this.executeSimple(sql);
            }
         }

         for(var i = 0; i < bookTags.length; i++)
         {
            var found = false;

            for(var j = 0; j < tags.length; j++)
            {
               if(bookTags[i] == tags[j].id)
               {
                  found = true;
                  break;
               }
            }

            if(found == false)
            {
               var sql = 'insert into book_2_tag values(' + id + ', ' + bookTags[i] + ')';
               this.executeSimple(sql);

               var sql = 'update tag set book_count = book_count + 1 where id = ' + bookTags[i];
               this.executeSimple(sql);
            }
         }
      },

      deleteBookTags: function(id)
      {
         var bookTags = this.getBookTags(id);

         for(var i = 0; i < bookTags.length; i++)
         {
            var sql = 'delete from book_2_tag where book_id = ' + bookTags[i].book_id;
            this.executeSimple(sql);

            var sql = 'update tag set book_count = book_count - 1 where id = ' + bookTags[i].tag_id;
            this.executeSimple(sql);
         }
      }
   },

   read: function(data, uri, content)
   {
      try
      {
         this.init(false);
         mylog.start();
         var name = this.store(data, uri, content);
         this.unzip();
         var id = this.getContent(name);
      }
      catch(e)
      {
         if(e == this.exception_encrypted ||
            e == this.exception_corrupted ||
            e == this.exception_too_large ||
            e == this.exception_invalid_characters ||
            e == this.exception_container_missing ||
            e == this.exception_opf_missing)
         {
            var exception = [];
            exception['code'] = e;

            throw(exception);
         }
         else
         {
            var exception = [];
            exception['code'] = this.exception_other;
            exception['text'] = mylog.getStack();

            throw(exception);
         }
      }

      mylog.log("end");
      return id;
   },

   getFileDir: function()
   {
      /* get epub id */
      var id = this.sqlite.getEpubId();

      /* get + save + create fileDir */
      var dir = this.epubRoot.clone();
      dir.append(id);
      this.fileDir = dir.clone();
      this.fileDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

      return this.fileDir;
   },

   store: function(data, uri, content)
   {
      mylog.log("store: begin");

      try
      {
         var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
         var dir = this.getFileDir();

         /* get file name */
         if(content)
         {
            content = unescape(content);
            var name = content.substring(content.lastIndexOf("=") + 1, content.lastIndexOf("."));

            if(name.lastIndexOf("'") != -1)
            {
               name = name.substr(name.lastIndexOf("'") + 1);
            }
         }
         else
         {
            uri = unescape(uri);
            var name = uri.substring(uri.lastIndexOf("/") + 1, uri.lastIndexOf("."));
         }

         file.initWithPath(dir.path);
         file.append(dir.leafName + ".epub");

         mylog.log("store: " + file.path);

         try
         {
            this.writeFile(file, data, false, false);
         }
         catch(e)
         {
            if(e.result == Components.results.NS_ERROR_XPC_BAD_CONVERT_JS)
            {
               e = this.exception_too_large;
            }
            else
            {
               mylog.log(e);
            }

            throw(e);
         }

         this.file = file;
      }
      catch(e)
      {
         mylog.log(e);
         throw(e);
      }

      mylog.log("store: end");
      return name;
   },

   unzip: function()
   {
      var itemFile =
      {
         get: function(file, fileDir)
         {
            var itemLocation = fileDir.clone();
            var parts = file.split("/");

            for (var i = 0; i < parts.length; ++i)
            {
               itemLocation.append(parts[i]);
            }

            return itemLocation;
         }
      }

      mylog.log("zip: begin");

      try
      {
         var zipReader = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(Ci.nsIZipReader);

         zipReader.open(this.file);
         zipReader.test(null);

         /* create directories first */
         var entries = zipReader.findEntries("*/");

         while(entries.hasMore())
         {
            var entryName = entries.getNext();
            var target = itemFile.get(entryName, this.fileDir);

            if(!target.exists())
            {
               target.create(Ci.nsILocalFile.DIRECTORY_TYPE, 0777);
            }
         }

         entries = zipReader.findEntries(null);

         while(entries.hasMore())
         {
            var entryName = entries.getNext();
            target = itemFile.get(entryName, this.fileDir);

            if(target.exists())
            {
               continue;
            }

            zipReader.extract(entryName, target);
         }

         zipReader.close();
      }
      catch(e)
      {
         if(e.result == Components.results.NS_ERROR_FILE_CORRUPTED)
         {
            e = this.exception_corrupted;
         }
         else if(e.result == Components.results.NS_ERROR_FILE_TARGET_DOES_NOT_EXIST)
         {
            e = this.exception_invalid_characters;
         }
         else
         {
            mylog.log(e);
         }

         throw(e);
      }

      mylog.log("zip: end");
   },

   readFile: function(file)
   {
      var charset = 'UTF-8';
      var buffer = {};

      mylog.log("readFile: begin, " + file.path);

      var fs = Cc['@mozilla.org/network/file-input-stream;1'].createInstance(Ci.nsIFileInputStream);
      var cs = Cc['@mozilla.org/intl/converter-input-stream;1'].createInstance(Ci.nsIConverterInputStream);

      fs.init(file, -1, -1, false);
      cs.init(fs, charset, fs.available(), cs.DEFAULT_REPLACEMENT_CHARACTER);
      cs.readString(fs.available(), buffer);
      var data = buffer.value;
      cs.close();
      fs.close();

      mylog.log("readFile: end");

      return data;
   },

   readFileBinary: function(file)
   {
      mylog.log("readFileBinary: begin, " + file.path);

      var fs = Cc['@mozilla.org/network/file-input-stream;1'].createInstance(Ci.nsIFileInputStream);
      var bs = Cc["@mozilla.org/binaryinputstream;1"].createInstance(Ci.nsIBinaryInputStream);

      fs.init(file, -1, -1, false);
      bs.setInputStream(fs);
      var data = bs.readBytes(bs.available());
      fs.close();

      mylog.log("readFileBinary: end");

      return data;
   },

   readURL: function(url)
   {
      var ioService=Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
      var scriptableStream=Cc["@mozilla.org/scriptableinputstream;1"].getService(Ci.nsIScriptableInputStream);

      var channel = ioService.newChannel(url, null, null);
      var input = channel.open();
      scriptableStream.init(input);
      var data = scriptableStream.read(input.available());
      scriptableStream.close();
      input.close();

      return data;
   },

   readURLBinary: function(url)
   {
      var ioService=Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
	  var binaryInputStream = Cc["@mozilla.org/binaryinputstream;1"].createInstance(Ci.nsIBinaryInputStream);

      var channel = ioService.newChannel(url, null, null);
      var input = channel.open();
      binaryInputStream.setInputStream(input);
      var data = binaryInputStream.readBytes(input.available());
      binaryInputStream.close();
      input.close();

      return data;
   },

   writeFile: function(file, data, append, char)
   {
      var MODE_WRONLY = 0x02;
      var MODE_CREATE = 0x08;
      var MODE_TRUNCATE = 0x20;
      var MODE_APPEND = 0x10;
      var flags = 0;
      var charset = 'UTF-8';

      if(append == true && file.exists())
      {
         flags = MODE_WRONLY | MODE_APPEND;
      }
      else
      {
         flags = MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE;
      }

      if(char == true)
      {
         var os = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
         var cs = Cc['@mozilla.org/intl/converter-output-stream;1'].createInstance(Ci.nsIConverterOutputStream);

         os.init(file, flags, 0644, 0);
         cs.init(os, charset, 0, 0);
         cs.writeString(data);
         cs.close();
         os.close();
      }
      else
      {
         var os = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);

         os.init(file, flags, 0644, 0);
         os.write(data, data.length);
         os.close();
      }
   },

   getEpubIdentifier: function(file)
   {
      function getData(is)
      {
         var charset = 'UTF-8';
         var buffer = {};

         var cs = Cc['@mozilla.org/intl/converter-input-stream;1'].createInstance(Ci.nsIConverterInputStream);

         cs.init(is, charset, is.available(), cs.DEFAULT_REPLACEMENT_CHARACTER);
         cs.readString(is.available(), buffer);
         var data = buffer.value;
         cs.close();

         return data;
      }

      try
      {
         var zipReader = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(Ci.nsIZipReader);
         var parser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);

         var identifier = null;

         zipReader.open(file);
         zipReader.test(null);

         var entries = zipReader.findEntries("META-INF/container.xml");

         while(entries.hasMore())
         {
            var entryName = entries.getNext();
            var is = zipReader.getInputStream(entryName);
            var data = getData(is);
            break;
         }

         if(!data)
         {
            var entries = zipReader.findEntries("meta-inf/container.xml");

            while(entries.hasMore())
            {
               var entryName = entries.getNext();
               var is = zipReader.getInputStream(entryName);
               var data = getData(is);
               break;
            }
         }

         data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');

         var container = parser.parseFromString(data, "text/xml");
         var rootfile = container.getElementsByTagNameNS("*", "rootfile")[0];
         var opffile = rootfile.getAttribute("full-path");

         var entries = zipReader.findEntries("*" + opffile);

         while(entries.hasMore())
         {
            var entryName = entries.getNext();
            var is = zipReader.getInputStream(entryName);
            var data = getData(is);
            break;
         }

         data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');

         var opf = parser.parseFromString(data, "text/xml");
         var package = opf.getElementsByTagNameNS("*", "package")[0];
         var metadata = opf.getElementsByTagNameNS("*", "metadata")[0];

         try
         {
            var identifierId = package.getAttribute("unique-identifier");
            var identifiers = metadata.getElementsByTagNameNS("*", "identifier");

            for(var i = 0; i < identifiers.length; i++)
            {
               if(identifiers[i].getAttribute("id") == identifierId)
               {
                  identifier = identifiers[i].childNodes[0].nodeValue;
                  break;
               }
            }
         }
         catch(e)
         {
         }

         if(!identifier || (identifier && identifier.match(/^[0-]+$/)))
         {
            mylog.log("getEpubIdentifier: no identifier. use file hash. epub: '" + file.path + "'.");
            identifier = this.getFileHash(file);
         }

         zipReader.close();
      }
      catch(e)
      {
         mylog.log("getEpubIdentifier: failed. epub: '" + file.path + "'. " + e);
      }

      return identifier;
   },

   getFileHash: function(file)
   {
      function toHexString(charCode)
      {
         return ("0" + charCode.toString(16)).slice(-2);
      }

      var is = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
      var ch = Cc["@mozilla.org/security/hash;1"].createInstance(Ci.nsICryptoHash);

      is.init(file, 0x01, 0444, 0);
      ch.init(ch.MD5);

      const PR_UINT32_MAX = 0xffffffff;
      ch.updateFromStream(is, PR_UINT32_MAX);

      var hash = ch.finish(false);
      var s = [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");

      return s;
   },

   getContent: function(name)
   {
      mylog.log("getContent: begin");

      var utils =
      {
         replaceCharacters: function(text)
         {
            var chars = [];
            var out = "";

            chars[''] = "a";
            chars[''] = "a";
            chars[''] = "a";
            chars[''] = "a";
            chars[''] = "ae";
            chars[''] = "ae";
            chars[''] = "o";
            chars[''] = "c";
            chars[''] = "e";
            chars[''] = "e";
            chars[''] = "e";
            chars[''] = "e";
            chars[''] = "i";
            chars[''] = "i";
            chars[''] = "i";
            chars[''] = "i";
            chars[''] = "n";
            chars[''] = "o";
            chars[''] = "o";
            chars[''] = "o";
            chars[''] = "oe";
            chars[''] = "oe";
            chars[''] = "ss";
            chars[''] = "u";
            chars[''] = "u";
            chars[''] = "u";
            chars[''] = "ue";
            chars[''] = "y";
            chars[''] = "y";
            chars['\''] = "";
            chars['`'] = "";
            chars[''] = "";

            text = text.toLowerCase();

            for(var i = 0; i < text.length; i++)
            {
               var char = text.substr(i, 1);
               var replace = chars[char];

               if(replace || replace == "")
               {
                  out = out + replace;
               }
               else
               {
                  out = out + char;
               }
            }

            out = out.replace(/[^a-z0-9_]/gi, "_");
            out = out.replace(/_{2,}/g, "_");

            return out;
         },

         getFileName: function(name, title, author)
         {
            var fileName = "";
            var underscore = [];

            if(title != "Unknown")
            {
               fileName = utils.replaceCharacters(title);

               if(author != "Unknown")
               {
                  fileName = fileName + "_" + utils.replaceCharacters(author);
               }

               fileName = fileName.substr(0, 50);
               underscore = fileName.match(/_/g);
            }

            if(fileName == "" ||
               (fileName != "" && underscore && underscore.length > fileName.length/3))
            {
               fileName = name;
               fileName = utils.replaceCharacters(fileName);
               fileName = fileName.substr(0, 50);
            }

            fileName = fileName + ".epub";

            return fileName;
         }
      }

      try
      {
         var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
         var parser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);

         var encrypted = false;

         file.initWithPath(this.fileDir.path);
         file.append("META-INF");
         file.append("encryption.xml");

         if(!file.exists())
         {
            file.initWithPath(this.fileDir.path);
            file.append("meta-inf");
            file.append("encryption.xml");
         }

         if(file.exists())
         {
            try
            {
               var data = this.readFile(file);
               data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');

               var encryption = parser.parseFromString(data, "text/xml");
               var encryptedData = encryption.getElementsByTagNameNS("*", "EncryptedData");

               for(var i = 0; i < encryptedData.length; i++)
               {
                  var keyInfo = encryptedData[i].getElementsByTagNameNS("*", "KeyInfo");

                  if(keyInfo.length > 0)
                  {
                     encrypted = true;
                     break;
                  }
               }
            }
            catch(e)
            {
            }
         }

         if(encrypted == true)
         {
            throw(this.exception_encrypted);
         }

         file.initWithPath(this.fileDir.path);
         file.append("META-INF");
         file.append("container.xml");

         if(!file.exists())
         {
            file.initWithPath(this.fileDir.path);
            file.append("meta-inf");
            file.append("container.xml");

            if(!file.exists())
            {
               throw(this.exception_container_missing);
            }
         }

         var data = this.readFile(file);

         data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');

         var container = parser.parseFromString(data, "text/xml");
         var rootfile = container.getElementsByTagNameNS("*", "rootfile")[0];
         var opffile = rootfile.getAttribute("full-path");

         var docRoot = this.fileDir.clone();
         var parts = opffile.split("/");

         for(var i = 0; i < parts.length; ++i)
         {
            if(i < parts.length - 1)
            {
               docRoot.append(parts[i]);
            }
         }

         this.docRoot = docRoot;

         var elements = opffile.split("/");
         var opffileName = elements[elements.length - 1];

         file.initWithPath(this.docRoot.path);
         file.append(opffileName);

         if(!file.exists())
         {
            throw(this.exception_opf_missing);
         }

         var opfData = this.readFile(file);
         opfData = opfData.replace(/xml version="1\.1"/, 'xml version="1.0"');

         var opf = parser.parseFromString(opfData, "text/xml");
         var package = opf.getElementsByTagNameNS("*", "package")[0];
         var metadata = opf.getElementsByTagNameNS("*", "metadata")[0];
         var manifest = opf.getElementsByTagNameNS("*", "manifest")[0];
         var spine = opf.getElementsByTagNameNS("*", "spine")[0];

         var toc = spine.getAttribute("toc");

         if(!toc)
         {
            toc = "ncx";
         }

         var itemref = spine.getElementsByTagNameNS("*", "itemref")[0];
         var idFirst = itemref.getAttribute("idref");

         var items = manifest.getElementsByTagNameNS("*", "item");

         for(var i = 0; i < items.length; i++)
         {
            var id = items[i].getAttribute("id");

            if(id == idFirst)
            {
               var hrefFirst = items[i].getAttribute("href");
            }
            else if(id == toc)
            {
               var hrefNcx = items[i].getAttribute("href");
            }
         }

         var epub = [];
         var identifier = null;

         try
         {
            var identifierId = package.getAttribute("unique-identifier");
            var identifiers = metadata.getElementsByTagNameNS("*", "identifier");

            for(var i = 0; i < identifiers.length; i++)
            {
               if(identifiers[i].getAttribute("id") == identifierId)
               {
                  identifier = identifiers[i].childNodes[0].nodeValue;
                  break;
               }
            }
         }
         catch(e)
         {
            dump("ops! no identifier.");
         }

         try
         {
            var title = metadata.getElementsByTagNameNS("*", "title")[0].childNodes[0].nodeValue;
         }
         catch(e)
         {
         }

         if(title)
         {
            epub['title'] = title;
         }
         else
         {
            epub['title'] = "Unknown";
         }

         try
         {
            var author = metadata.getElementsByTagNameNS("*", "creator")[0].getAttribute("opf:file-as");
         }
         catch(e)
         {
         }

         if(!author)
         {
            try
            {
               var author = metadata.getElementsByTagNameNS("*", "creator")[0].childNodes[0].nodeValue;
            }
            catch(e)
            {
            }
         }

         if(author)
         {
            epub['author'] = author;
         }
         else
         {
            epub['author'] = "Unknown";
         }

         var fileName = utils.getFileName(name, epub['title'], epub['author']);
         epub['name'] = fileName;

         epub['path'] = this.fileDir.leafName;
         epub['first'] = hrefFirst;

         var file = this.docRoot.clone();
         var url = "file:///" + file.path.replace(/\\/g, "/");
         epub['url'] = url;

         var title = epub['title'].replace(/["']/g, "") + " - " + epub['author'].replace(/["']/g, "");

         if(!identifier || (identifier && identifier.match(/^[0-]+$/)))
         {
            file = this.epubRoot.clone();
            file.append(epub['path']);
            file.append(epub['path'] + ".epub");
            var hash = this.getFileHash(file);

            identifier = hash;
         }

         epub['identifier'] = identifier;

         this.createNav(hrefNcx, opf);

         this.sqlite.init(this.epubRoot, false);
         var id = this.sqlite.storeEpub(epub);

         var count = this.storeMetadata(id, metadata);
         this.sqlite.storeBookMetadata(id, count);

         mylog.log("getContent: end");
         return id;
      }
      catch(e)
      {
         mylog.log(e);
         throw(e);
      }
   },

   storeMetadata: function(id, metadata)
   {
      const meta_none = 0;
      const meta_title = 1;
      const meta_creator = 2;
      const meta_subject = 3;
      const meta_description = 4;
      const meta_publisher = 5;
      const meta_contributor = 6;
      const meta_date = 7;
      const meta_type = 8;
      const meta_format = 9;
      const meta_identifier = 10;
      const meta_source = 11;
      const meta_language = 12;
      const meta_relation = 13;
      const meta_coverage = 14;
      const meta_rights = 15;

      var counter = 0;

      try
      {
         this.sqlite.deleteMetadata(id);

         var childNodes = metadata.childNodes;

         for(var i = 0; i < childNodes.length; i++)
         {
            if(childNodes[i].nodeType == metadata.ELEMENT_NODE)
            {
               var data = null;

               try
               {
                  data = childNodes[i].childNodes[0].nodeValue;
               }
               catch(e)
               {
               }

               if(data)
               {
                  data = data.replace(/"/g, "'");

                  var type = meta_none;
                  var subtype = null;

                  if(childNodes[i].nodeName == "dc:title")
                  {
                     type = meta_title;
                  }
                  else if(childNodes[i].nodeName == "dc:creator")
                  {
                     type = meta_creator;
                  }
                  else if(childNodes[i].nodeName == "dc:subject")
                  {
                     type = meta_subject;
                  }
                  else if(childNodes[i].nodeName == "dc:description")
                  {
                     type = meta_description;
                  }
                  else if(childNodes[i].nodeName == "dc:publisher")
                  {
                     type = meta_publisher;
                  }
                  else if(childNodes[i].nodeName == "dc:contributor")
                  {
                     type = meta_contributor;
                     subtype = childNodes[i].getAttribute("opf:role");
                  }
                  else if(childNodes[i].nodeName == "dc:date")
                  {
                     type = meta_date;
                     subtype = childNodes[i].getAttribute("opf:event");
                  }
                  else if(childNodes[i].nodeName == "dc:type")
                  {
                     type = meta_type;
                  }
                  else if(childNodes[i].nodeName == "dc:format")
                  {
                     type = meta_format;
                  }
                  else if(childNodes[i].nodeName == "dc:identifier")
                  {
                     type = meta_identifier;
                     subtype = childNodes[i].getAttribute("opf:scheme");
                  }
                  else if(childNodes[i].nodeName == "dc:source")
                  {
                     type = meta_source;
                  }
                  else if(childNodes[i].nodeName == "dc:language")
                  {
                     type = meta_language;
                  }
                  else if(childNodes[i].nodeName == "dc:relation")
                  {
                     type = meta_relation;
                  }
                  else if(childNodes[i].nodeName == "dc:coverage")
                  {
                     type = meta_coverage;
                  }
                  else if(childNodes[i].nodeName == "dc:rights")
                  {
                     type = meta_rights;
                  }

                  if(type != meta_none)
                  {
                     var meta = [];
                     meta['id'] = id;
                     meta['type'] = type;

                     if(!subtype)
                     {
                        subtype = "";
                     }

                     meta['subtype'] = subtype;
                     meta['data'] = data;

                     this.sqlite.storeMetadata(meta);
                     counter++;
                  }
               }
            }
         }
      }
      catch(e)
      {
         mylog.log(e);
      }

      return counter;
   },

   getMetadata: function(id)
   {
      var parser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);
      var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
      var meta = null;

      try
      {
         this.init(false);
         var epub = this.sqlite.getEpub(id);

         if(!epub['metadata'])
         {
            file.initWithPath(this.epubRoot.path);
            file.append(epub['path']);
            file.append("META-INF");
            file.append("container.xml");

            if(!file.exists())
            {
               file.initWithPath(this.epubRoot.path);
               file.append(epub['path']);
               file.append("meta-inf");
               file.append("container.xml");
            }

            var data = this.readFile(file);

            data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');

            var container = parser.parseFromString(data, "text/xml");
            var rootfile = container.getElementsByTagNameNS("*", "rootfile")[0];
            var opffile = rootfile.getAttribute("full-path");

            var docRoot = this.epubRoot.clone();
            docRoot.append(epub['path']);
            var parts = opffile.split("/");

            for(var i = 0; i < parts.length; ++i)
            {
               if(i < parts.length - 1)
               {
                  docRoot.append(parts[i]);
               }
            }

            var elements = opffile.split("/");
            var opffileName = elements[elements.length - 1];

            file.initWithPath(docRoot.path);
            file.append(opffileName);

            var opfData = this.readFile(file);
            opfData = opfData.replace(/xml version="1\.1"/, 'xml version="1.0"');

            var opf = parser.parseFromString(opfData, "text/xml");
            var metadata = opf.getElementsByTagNameNS("*", "metadata")[0];

            var count = this.storeMetadata(id, metadata);
            this.sqlite.storeBookMetadata(id, count);
         }

         var meta = this.sqlite.getMetadata(id);
      }
      catch(e)
      {
         mylog.log(e);
      }

      return meta;
   },

   createNav: function(hrefNcx, opf)
   {
      var html =
      {
         get: function(pointsToc, pointsSpine, epubRoot, error, template)
         {
            var root = "file://" + epubRoot.path.replace(/\\/g, "/");
            template = template.replace(/%root%/, root);

            var repeat_begin = "%repeat-begin-toc%";
            var repeat_end = "%repeat-end-toc%";
            var repeat = template.substring(template.search(new RegExp(repeat_begin)) + repeat_begin.length, template.search(new RegExp(repeat_end)));

            var error_begin = "%begin-error%";
            var error_end = "%end-error%";
            var errorTemplate = template.substring(template.search(new RegExp(error_begin)) + error_begin.length, template.search(new RegExp(error_end)));

            var content = "";

            if(error == true)
            {
               var msg = this.getProperty("core.info.error_toc");
               var element = errorTemplate;

               element = element.replace(/%nav_error%/, msg);

               content = content + element;
            }

            for(var i = 0; i < pointsToc.length; i++)
            {
               var spacer = "";
               var fontSize;

               for(var j = 1; j < pointsToc[i]['level']; j++)
               {
                  spacer = spacer + "&nbsp;&nbsp;&nbsp;";
               }

               if(pointsToc[i].level == 1)
               {
                  var level = "topLevel";
               }
               else
               {
                  var level = "childLevel";
               }

               var element = repeat;
               element = element.replace(/%spacer%/, spacer);
               element = element.replace(/%level%/, level);
               element = element.replace(/%indent%/, ((pointsToc[i]['level'] - 1) * 15) + 10);
               element = element.replace(/%url%/g, pointsToc[i]['content']);
               element = element.replace(/%desc%/, this.makeHtmlSave(pointsToc[i]['label']));

               content = content + element;
            }

            template = template.replace(/%begin-error%(.|\s)*%repeat-end-toc%/, content);

            var repeat_begin = "%repeat-begin-spine%";
            var repeat_end = "%repeat-end-spine%";
            var repeat = template.substring(template.search(new RegExp(repeat_begin)) + repeat_begin.length, template.search(new RegExp(repeat_end)));
            var content = "";

            for(var i = 0; i < pointsSpine.length; i++)
            {
               var element = repeat;
               element = element.replace(/%url%/, pointsSpine[i]['content']);
               element = element.replace(/%desc%/, this.makeHtmlSave(pointsSpine[i]['label']));
               element = element.replace(/%id%/g, i + 1);

               content = content + element;
            }

            template = template.replace(/%repeat-begin-spine%(.|\s)*%repeat-end-spine%/, content);

            return template;
         },

         makeHtmlSave: function(text)
         {
            if(text)
            {
               text = text.replace(/</g, "&lt;");
               text = text.replace(/>/g, "&gt;");
            }

            return text;
         },

         getProperty: function(name)
         {
            var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                         .getService(Components.interfaces.nsIStringBundleService)
                         .createBundle("chrome://epubreader/locale/epubreader.properties");

            var property = bundle.GetStringFromName(name);

            return property;
         }
      }

      mylog.log("createNav: begin");
      var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
      var parser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);
      var error = false;

      if(hrefNcx)
      {
         var dir = this.docRoot.clone();
         var parts = hrefNcx.split("/");

         for(var i = 0; i < parts.length; i++)
         {
           dir.append(parts[i]);
         }

         try
         {
            file.initWithPath(dir.path);
            var data = this.readFile(file);

            data = data.replace(/xml version="1\.1"/, 'xml version="1.0"');
            data = data.replace(/&(?!amp;)([a-z]+;)/gi, "&amp;$1");
            data = data.replace(/&(?![a-z]+;)/gi, "&amp;");

            var nav = parser.parseFromString(data, "text/xml");
            var navMap = nav.getElementsByTagNameNS("*", "navMap")[0];
            var navPoints = navMap.getElementsByTagNameNS("*", "navPoint");
         }
         catch(e)
         {
            mylog.log(e);

            if(e.result != Components.results.NS_ERROR_FILE_NOT_FOUND)
            {
               error = true;
            }
         }
      }

      var pointsToc = [];

      if(navPoints && navPoints.length > 0)
      {
         var pointCounter = 0;

         for(var i = 0; i < navPoints.length; i++)
         {
            try
            {
               var label = navPoints[i].getElementsByTagNameNS("*", "navLabel")[0]
                           .getElementsByTagNameNS("*", "text")[0].textContent;

               var content = navPoints[i].getElementsByTagNameNS("*", "content")[0].getAttribute("src");
               var navPoint = navPoints[i];
               var point = [];

               for(var j = 1; j < 10; j++)
               {
                  var parent = navPoint.parentNode;

                  if(!parent.nodeName.match("navPoint"))
                  {
                     point['level'] = j;
                     break;
                  }
                  else
                  {
                     navPoint = parent;
                  }
               }

               point['label'] = label;
               point['content'] = content;

               pointsToc[pointCounter] = point;
               pointCounter++;
            }
            catch(e)
            {
            }
         }
      }
      else
      {
         var spine = opf.getElementsByTagNameNS("*", "spine")[0];
         var itemrefs = spine.getElementsByTagNameNS("*", "itemref");

         var manifest = opf.getElementsByTagNameNS("*", "manifest")[0];
         var items = manifest.getElementsByTagNameNS("*", "item");

         var pointCounter = 0;

         for(var i = 0; i < itemrefs.length; i++)
         {
            for(var j = 0; j < items.length; j++)
            {
               var idref = itemrefs[i].getAttribute("idref");
               var id = items[j].getAttribute("id");

               if(idref == id)
               {
                  var href = items[j].getAttribute("href");
                  var point = [];

                  point['label'] = (i+1).toString() + ".";
                  point['content'] = href;
                  point['level'] = 1;

                  pointsToc[pointCounter] = point;
                  pointCounter++;
                  break;
               }
            }
         }
      }

      var spine = opf.getElementsByTagNameNS("*", "spine")[0];
      var itemrefs = spine.getElementsByTagNameNS("*", "itemref");

      var manifest = opf.getElementsByTagNameNS("*", "manifest")[0];
      var items = manifest.getElementsByTagNameNS("*", "item");

      var pointsSpine = [];
      var pointCounter = 0;

      for(var i = 0; i < itemrefs.length; i++)
      {
         for(var j = 0; j < items.length; j++)
         {
            var idref = itemrefs[i].getAttribute("idref");
            var id = items[j].getAttribute("id");

            if(idref == id)
            {
               var href = items[j].getAttribute("href");
               var point = [];

               point['label'] = "Navpoint " + (i+1);
               point['content'] = href;

               pointsSpine[pointCounter] = point;
               pointCounter++;
               break;
            }
         }
      }

      var template = this.readURL("chrome://epubreader/content/nav_template.html");
      var content = html.get(pointsToc, pointsSpine, this.epubRoot, error, template);

      file = this.docRoot.clone();
      file.append("nav.html");
      this.writeFile(file, content, false, true);
      mylog.log("createNav: end");
   },

   getEpubList: function(tag_id)
   {
      const sort_none = 0;
      const sort_title_asc = 1;
      const sort_title_desc = 2;
      const sort_author_asc = 3;
      const sort_author_desc = 4;
      const sort_timestamp_asc = 5;
      const sort_timestamp_desc = 6;

      var html =
      {
         get: function(epubs, epubRoot, template)
         {
            var repeat_begin = "%repeat-begin%";
            var repeat_end = "%repeat-end%";
            var repeat = template.substring(template.search(new RegExp(repeat_begin)) + repeat_begin.length, template.search(new RegExp(repeat_end)));
            var content = "";

            for(var i = 0; i < epubs.length; i++)
            {
               var element = repeat;

               element = element.replace(/%title%/, this.makeHtmlSave(epubs[i]['title']));
               element = element.replace(/%author%/, this.makeHtmlSave(epubs[i]['author']));

               var date = new Date((epubs[i]['timestamp']*1000));
               var timestamp = date.toLocaleFormat("%x");
               element = element.replace(/%timestamp%/, timestamp);

               element = element.replace(/%identifier%/, epubs[i]['identifier']);
               element = element.replace(/%id%/g, epubs[i]['id']);

               var url = "file:///" + epubRoot.path.replace(/\\/g, "/") + "/" + epubs[i]['path'] + "/" + epubs[i]['name'];
               element = element.replace(/%file%/g, url);

               content = content + element;
            }

            template = template.replace(/%repeat-begin%(.|\s)*%repeat-end%/, content);

            return template;
         },

         makeHtmlSave: function(text)
         {
            if(text)
            {
               text = text.replace(/</g, "&lt;");
               text = text.replace(/>/g, "&gt;");
               text = text.replace(/&/g, "&amp;");
               text = text.replace(/"/g, "&quot;");
               text = text.replace(/'/g, "&apos;");
            }

            return text;
         }
      }

      function compare(a, b)
      {
         if(sort == sort_title_asc)
         {
            return a['title'].localeCompare(b['title']);
         }
         else if(sort == sort_title_desc)
         {
            return b['title'].localeCompare(a['title']);
         }
         else if(sort == sort_author_asc)
         {
            return a['author'].localeCompare(b['author']);
         }
         else if(sort == sort_author_desc)
         {
            return b['author'].localeCompare(a['author']);
         }
         else if(sort == sort_timestamp_asc)
         {
            if(a['timestamp'] > b['timestamp'])
            {
               return 1;
            }
            else if(a['timestamp'] < b['timestamp'])
            {
               return -1;
            }
            else
            {
               return 0;
            }
         }
         else if(sort == sort_timestamp_desc)
         {
            if(a['timestamp'] < b['timestamp'])
            {
               return 1;
            }
            else if(a['timestamp'] > b['timestamp'])
            {
               return -1;
            }
            else
            {
               return 0;
            }
         }
         else
         {
            return 0;
         }
      }

      var pref = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService)
                 .getBranch("extensions.epubreader.");

      var sort = sort_none;

      this.init(false);

      var epubs = this.sqlite.getEpubs(tag_id);

      try
      {
         sort = pref.getIntPref("library_sort");
      }
      catch(e)
      {
      }

      if(sort != sort_none)
      {
         epubs.sort(compare);
      }

      var template = this.readURL("chrome://epubreader/content/library_template.html");
      var content = html.get(epubs, this.epubRoot, template);

      return content;
   },

   deleteEpub: function(id)
   {
      try
      {
         this.init(false);
         mylog.log("-----------------------------------");
         mylog.log("deleteEpub: begin");
         var root = this.epubRoot;
         var path = this.sqlite.deleteEpub(id);
         var fileDir = root.clone();
         fileDir.append(path);
         mylog.log("deleteEpub: delete " + fileDir.path);
         fileDir.remove(true);
         mylog.log("deleteEpub: end");
      }
      catch(e)
      {
         mylog.log("deleteEpub: failed. " + e);
         mylog.log("deleteEpub: end");
         throw(e);
      }
   },

   deleteEpubPermanent: function(id)
   {
      try
      {
         this.init(false);
         mylog.log("-----------------------------------");
         mylog.log("deleteEpubPermanent: begin");
         mylog.log("deleteEpubPermanent: delete " + id);
         var path = this.sqlite.deleteEpubPermanent(id);
         mylog.log("deleteEpubPermanent: end");
      }
      catch(e)
      {
         mylog.log("deleteEpubPermanent: failed. " + e);
         mylog.log("deleteEpubPermanent: end");
         throw(e);
      }
   },

   deleteEpubsPermanent: function()
   {
      try
      {
         this.init(false);
         mylog.log("-----------------------------------");
         mylog.log("deleteEpubsPermanent: begin");
         var path = this.sqlite.deleteEpubsPermanent();
         mylog.log("deleteEpubsPermanent: end");
      }
      catch(e)
      {
         mylog.log("deleteEpubsPermanent: failed. " + e);
         mylog.log("deleteEpubsPermanent: end");
         throw(e);
      }
   },

   updateCSS: function()
   {
      this.init(false);
      this.copyCSS();
   },

   getEpub: function(id)
   {
      this.init(false);
      var epub = this.sqlite.getEpub(id);

      return epub;
   },

   getEpubs: function()
   {
      this.init(false);
      var epubs = this.sqlite.getEpubs();

      return epubs;
   },

   getDeletedEpubs: function()
   {
      this.init(false);
      var epubs = this.sqlite.getDeletedEpubs();

      return epubs;
   },

   getEpubByIdentifier: function(id)
   {
      this.init(false);
      var epub = this.sqlite.getEpubByIdentifier(id);

      return epub;
   },

   getEpubRoot: function()
   {
      this.init(false);
      return this.epubRoot.path;
   },

   storeBookmark: function(bookmark)
   {
      this.init(false);
      this.sqlite.storeBookmark(bookmark);
   },

   getBookmarks: function(id)
   {
      this.init(false);
      var bookmarks = this.sqlite.getBookmarks(id);

      return bookmarks;
   },

   getBookmarkByName: function(id, name)
   {
      this.init(false);
      var bookmark = this.sqlite.getBookmarkByName(id, name);

      return bookmark;
   },

   storeCatalog: function(catalog)
   {
      this.init(false);
      this.sqlite.storeCatalog(catalog);
   },

   getCatalogs: function()
   {
      this.init(false);
      var catalogs = this.sqlite.getCatalogs();

      return catalogs;
   },

   getCatalogByName: function(name)
   {
      this.init(false);
      var catalog = this.sqlite.getCatalogByName(name);

      return catalog;
   },

   storeTag: function(name)
   {
      this.init(false);
      var ret = this.sqlite.storeTag(name);

      return ret;
   },

   updateTag: function(id, name)
   {
      this.init(false);
      var ret = this.sqlite.updateTag(id, name);

      return ret;
   },

   deleteTag: function(id)
   {
      this.init(false);
      this.sqlite.deleteTag(id);
   },

   getTags: function()
   {
      this.init(false);
      var tags = this.sqlite.getTags();

      return tags;
   },

   getBookTags: function(id)
   {
      this.init(false);
      var tags = this.sqlite.getBookTags(id);

      return tags;
   },

   storeBookTags: function(id, bookTags)
   {
      this.init(false);
      this.sqlite.storeBookTags(id, bookTags);
   },

   importEpub: function(file)
   {
      this.init(false);
      mylog.log("-----------------------------------");
      mylog.log("importEpub: begin");

      var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
      var uri = ios.newFileURI(file);

      try
      {
         var data = this.readFileBinary(file);
         mylog.log("importEpub: import " + file.leafName);
         this.read(data, uri.spec, null);
         mylog.log("importEpub: end");
      }
      catch(e)
      {
         mylog.log("importEpub: failed. " + e);
         mylog.log("importEpub: end");
         throw(e);
      }
   },

   exportEpub: function(id, dir)
   {
      this.init(false);
      mylog.log("-----------------------------------");
      mylog.log("exportEpub: begin");

      var pref = Cc["@mozilla.org/preferences-service;1"]
                 .getService(Ci.nsIPrefService).getBranch("extensions.epubreader.");

      var source = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
      var target = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
      var test = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);

      var epub = this.sqlite.getEpub(id);

      try
      {
         source = this.epubRoot.clone();
         source.append(epub['path']);
         source.append(epub['path'] + ".epub");

         var name = epub['name'];

         test.initWithPath(dir);
         test.append(name);

         if(test.exists())
         {
            name = name.match(/(.*)\.epub/)[1];
            name = name + "_2.epub";
         }

         target.initWithPath(dir);

         mylog.log("exportEpub: source: " + source.path + ", target path: " + target.path + ", target file: " + name);
         source.copyTo(target, name);
         mylog.log("exportEpub: end");
      }
      catch(e)
      {
         mylog.log("exportEpub: failed. " + e);
         mylog.log("exportEpub: end");
         throw(e);
      }
   },

   upgradeTasks: function(oldVersion, newVersion)
   {
      this.init(true);

      mylog.log("oldVersion: " + oldVersion + ", newVersion: " + newVersion);

      var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                           .getService(Components.interfaces.nsIVersionComparator);

      /* old version < 1.2.1 and new version >= 1.2.1 */
      if(versionChecker.compare(oldVersion, "1.2.1") < 0 &&
         versionChecker.compare(newVersion, "1.2.1") >= 0)
      {
         mylog.log("old version < 1.2.1 and new version >= 1.2.1");

         var sql = "drop table book";
         this.sqlite.executeSimple(sql);
         this.sqlite.createEpubTables();
      }

      /* old version < 1.2.5 and new version >= 1.2.5 */
      if(versionChecker.compare(oldVersion, "1.2.5") < 0 &&
         versionChecker.compare(newVersion, "1.2.5") >= 0)
      {
         mylog.log("old version < 1.2.5 and new version >= 1.2.5");

         var sql = "alter table book add identifier text";

         try
         {
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }

         var sql = "update book set identifier = '-'";
         this.sqlite.executeSimple(sql);
      }

      /* old version < 1.2.6 and new version >= 1.2.6 */
      if(versionChecker.compare(oldVersion, "1.2.6") < 0 &&
         versionChecker.compare(newVersion, "1.2.6") >= 0)
      {
         mylog.log("old version < 1.2.6 and new version >= 1.2.6");

         var entries = this.epubRoot.directoryEntries;

         while(entries.hasMoreElements())
         {
            var entry = entries.getNext();
            entry.QueryInterface(Components.interfaces.nsIFile);

            if(entry.isDirectory())
            {
               var files = entry.directoryEntries;

               while(files.hasMoreElements())
               {
                  var file = files.getNext();
                  file.QueryInterface(Components.interfaces.nsIFile);

                  if(file.path.match(/.*\.epub/))
                  {
                     var name = entry.leafName + ".epub";

                     if(file.leafName != name)
                     {
                        file.moveTo(null, name);
                     }
                  }
               }
            }
         }
      }
      /* old version < 1.2.7.0 and new version >= 1.2.7.0 */
      if(versionChecker.compare(oldVersion, "1.2.7.0") < 0 &&
         versionChecker.compare(newVersion, "1.2.7.0") >= 0)
      {
         mylog.log("old version < 1.2.7.0 and new version >= 1.2.7.0");

         try
         {
            var sql = "drop table catalog";
            this.sqlite.executeSimple(sql);
            this.sqlite.createEpubTables();
         }
         catch(e)
         {
         }
      }

      /* old version < 1.2.8.0 and new version >= 1.2.8.0 */
      if(versionChecker.compare(oldVersion, "1.2.8.0") < 0 &&
         versionChecker.compare(newVersion, "1.2.8.0") >= 0)
      {
         mylog.log("old version < 1.2.8.0 and new version >= 1.2.8.0");

         try
         {
            var sql = "alter table book add timestamp integer";
            this.sqlite.executeSimple(sql);

            var sql = "update book set timestamp = strftime('%s','now')";
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }

         try
         {
            var sql = "alter table book add status integer";
            this.sqlite.executeSimple(sql);

            var sql = "update book set status = 1";
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }
      }

      /* old version < 1.2.9.0 and new version >= 1.2.9.0 */
      if(versionChecker.compare(oldVersion, "1.2.9.0") < 0 &&
         versionChecker.compare(newVersion, "1.2.9.0") >= 0)
      {
         mylog.log("old version < 1.2.9.0 and new version >= 1.2.9.0");

         try
         {
            var sql = "alter table book add metadata integer";
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }
      }

      /* old version < 1.4.0.0 and new version >= 1.4.0.0 */
      if(versionChecker.compare(oldVersion, "1.4.0.0") < 0 &&
         versionChecker.compare(newVersion, "1.4.0.0") >= 0)
      {
         mylog.log("old version < 1.4.0.0 and new version >= 1.4.0.0");

         try
         {
            var sql = "drop table bookmark";
            this.sqlite.executeSimple(sql);
            this.sqlite.createEpubTables();
         }
         catch(e)
         {
         }
      }

      /* old version < 1.4.1.0 and new version >= 1.4.1.0 */
      if(versionChecker.compare(oldVersion, "1.4.1.0") < 0 &&
         versionChecker.compare(newVersion, "1.4.1.0") >= 0)
      {
         mylog.log("old version < 1.4.1.0 and new version >= 1.4.1.0");

         try
         {
            var sql = "delete from catalog where code = 'f7be809b8ae51e4e01747a081fa57cf0'";
            this.sqlite.executeSimple(sql);

            var sql = "delete from catalog where code = '5fc45c298739e4a7d771ee321b557d6d'";
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }
      }

      /* old version <= 1.4.1.1 and new version > 1.4.1.1 */
      if(versionChecker.compare(oldVersion, "1.4.1.1") <= 0 &&
         versionChecker.compare(newVersion, "1.4.1.1") > 0)
      {
         mylog.log("old version <= 1.4.1.1 and new version > 1.4.1.1");

         try
         {
            var sql = "delete from catalog where code = 'ef57c8cbcb29d1453265e807bf91afdf'";
            this.sqlite.executeSimple(sql);
         }
         catch(e)
         {
         }
      }

      this.sqlite.fillEpubTables();
   }
}

/* epubreader component */
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const EPUBREADER_CONVERSION = "?from=application/epub+zip&to=*/*";
const EPUBREADER_CONTRACT_ID = "@mozilla.org/streamconv;1" + EPUBREADER_CONVERSION;
const EPUBREADER_COMPONENT_ID = Components.ID("{0b4affe0-b71e-11de-8a39-0800200c9a66}");

function EPUBReader()
{
   this.wrappedJSObject = this;
};

EPUBReader.prototype =
{
   classDescription: "EPUBREADER XPCOM Component",
   classID: EPUBREADER_COMPONENT_ID,
   contractID: EPUBREADER_CONTRACT_ID,

   _xpcom_categories:
   [{
       category: "@mozilla.org/streamconv;1",
       entry: EPUBREADER_CONVERSION,
       value: "EPUB to HTML stream converter"
    },
    {
       category: "net-content-sniffers"
    }],

   QueryInterface: XPCOMUtils.generateQI
   ([
       Ci.nsISupports,
       Ci.nsIStreamConverter,
       Ci.nsIStreamListener,
       Ci.nsIRequestObserver,
       Ci.nsIContentSniffer
   ]),

   data: null,
   listener: null,
   channel: null,
   openFrom: null,
   currentUri: null,
   contentDisposition: null,

   /* nsIContentSniffer */
   getMIMETypeFromContent: function(request, data, length)
   {
      function checkMimetype(data)
      {
         if(data.length < 59)
         {
            return false;
         }

         var type = [109,105,109,101,116,121,112,101,97,112,112,108,105,99,97,116,105,111,110,47,101,112,117,98,43,122,105,112];
         var offset = 30;

         for(var i = 0; i < type.length; i++)
         {
            if(data[i + offset] != type[i])
            {
               return false;
            }
         }

         return true;
      }

      EPUBReader.contentDisposition = null;

      if(request instanceof Ci.nsIHttpChannel)
      {
         var chan = request.QueryInterface(Ci.nsIChannel);
         var httpChannel = chan.QueryInterface(Ci.nsIHttpChannel);

         try
         {
            var uri = request.QueryInterface(Ci.nsIChannel).URI.spec;
            var isEpub = false;

            try
            {
               var type = httpChannel.getResponseHeader("Content-Type");
            }
            catch(e)
            {
            }

            if(type && type.match(/application\/epub\+zip/g))
            {
               isEpub = true;
            }
            else if(uri.match(/.*[^=]\.epub$/g))
            {
               isEpub = true;
            }
            else
            {
               try
               {
                  var disposition = httpChannel.getResponseHeader("Content-Disposition");
               }
               catch(e)
               {
               }

               if(disposition && disposition.match(/.*\.epub/g))
               {
                  isEpub = true;
               }
            }

            if(isEpub)
            {
               if(checkMimetype(data))
               {
                  try
                  {
                     var disposition = httpChannel.getResponseHeader("Content-Disposition");
                     var parts = disposition.match(/(.*;.*)(filename.*)/i);
                     var header = "";

                     if(parts && parts[2])
                     {
                        header = parts[2];
                     }

                     httpChannel.setResponseHeader("Content-Disposition", header, false);
                     EPUBReader.contentDisposition = disposition;
                  }
                  catch(e)
                  {
                  }

                  EPUBReader.openFrom = "web";
                  return "application/epub+zip";
               }
               else
               {
               }
            }
         }
         catch(e)
         {
         }
      }
      else if(request instanceof Ci.nsIFileChannel)
      {
         var uri = request.QueryInterface(Ci.nsIChannel).URI.spec;

         if(uri.match(/.*\.epub$/g))
         {
            if(epub.app == epub.app_firefox || epub.app == epub.app_seamonkey)
            {
               var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
               var browser = wm.getMostRecentWindow("navigator:browser").getBrowser();
               EPUBReader.currentUri = browser.getBrowserForTab(browser.selectedTab).contentWindow.location.href;
               EPUBReader.openFrom = "local";
            }

            return "application/epub+zip";
         }
      }
   },
   /* nsIContentSniffer end */

   /* nsIStreamConverter */
   convert: function(fromStream, fromType, toType, ctxt)
   {
      return fromStream;
   },

   asyncConvertData: function(fromType, toType, listener, ctxt)
   {
      this.listener = listener;
   },
   /* nsIStreamConverter end */

   /* nsIStreamListener */
   onDataAvailable: function(request, context, inputStream, offset, count)
   {
		var binaryInputStream = Cc["@mozilla.org/binaryinputstream;1"].createInstance(Ci.nsIBinaryInputStream);

		binaryInputStream.setInputStream(inputStream);
        this.data += binaryInputStream.readBytes(count);
   },
   /* nsIStreamListener end */

   /* nsIRequestObserver */
   onStartRequest: function(request, context)
   {
      this.data = "";

      /* sets charset if it is available. for documents loaded from the filesystem, this is not set. */
      this.charset = request.QueryInterface(Ci.nsIChannel).contentCharset || 'UTF-8';

      this.channel = request;
      this.channel.contentType = "text/html";
      this.channel.contentCharset = "UTF-8";

      if(this.channel instanceof Ci.nsIHttpChannel)
      {
         var chan = request.QueryInterface(Ci.nsIChannel);
         var httpChannel = chan.QueryInterface(Ci.nsIHttpChannel);
         httpChannel.setResponseHeader("Content-Disposition", "", false);
      }

      this.listener.onStartRequest(this.channel, context);
   },

   onStopRequest: function(request, context, statusCode)
   {
      var browser =
      {
         redirect: function(id, openFrom, currentUri)
         {
            if(epub.app == epub.app_firefox || epub.app == epub.app_seamonkey)
            {
               var url = "chrome://epubreader/content/reader.xul?id=" + id;
               var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
               var browser = wm.getMostRecentWindow("navigator:browser").getBrowser();
               var currentTab = browser.selectedTab;

               var baseUrl = url.substr(0, url.lastIndexOf("?"));
               var regExp = new RegExp(baseUrl + ".*");

               var tab = null;
               var numTabs = browser.browsers.length;

               for(var i = 0; i < numTabs; i++)
               {
                  var currentBrowser = browser.getBrowserAtIndex(i);

                  if(unescape(currentBrowser.currentURI.spec).match(regExp))
                  {
                     tab = browser.tabContainer.childNodes[i];
                  }
               }

               if(tab || (!tab && openFrom == "local" && currentUri.match(regExp)))
               {
                  browser.selectedTab = tab;
                  var nav = browser.selectedBrowser.webNavigation;
                  nav.loadURI(url, nav.LOAD_FLAGS_NONE, null, null, null);
               }
               else
               {
                  tab = browser.addTab(url);
                  browser.selectedTab = tab;
               }

               if(browser.selectedTab != currentTab)
               {
                  browser.getBrowserForTab(currentTab).contentWindow.history.back();
               }
            }
            else if(epub.app == epub.app_fennec)
            {
               var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
               var browserEnumerator = wm.getEnumerator("navigator:browser");
               var browserWin = browserEnumerator.getNext();
               var browser = browserWin.Browser;
               var url = "chrome://epubreader/content/reader.xul?id=" + id;
               browser.selectedBrowser.loadURI(url, browser.LOAD_FLAGS_NONE, null, null, null);
            }
         }
      }

      var html =
      {
         get: function(e)
         {
            if(e['code'] == epub.exception_encrypted)
            {
               var message = this.getProperty("core.info.encrypted");
               var exception = "";
            }
            else if(e['code'] == epub.exception_corrupted)
            {
               var message = this.getProperty("core.error.corrupted");
               var exception = "";
            }
            else if(e['code'] == epub.exception_too_large)
            {
               var message = this.getProperty("core.error.too_large");
               var exception = "";
            }
            else if(e['code'] == epub.exception_invalid_characters)
            {
               var message = this.getProperty("core.error.invalid_characters");
               var exception = "";
            }
            else if(e['code'] == epub.exception_container_missing)
            {
               var message = this.getProperty("core.error.container_missing");
               var exception = "";
            }
            else if(e['code'] == epub.exception_opf_missing)
            {
               var message = this.getProperty("core.error.opf_missing");
               var exception = "";
            }
            else if(e['code'] == epub.exception_http_status_not_authorized)
            {
               var message = this.getProperty("core.error.not_authorized");
               var exception = "";
            }
            else if(e['code'] == epub.exception_http_status_not_found)
            {
               var message = this.getProperty("core.error.not_found");
               var exception = "";
            }
            else if(e['code'] == epub.exception_http_status_other)
            {
               var message = this.getProperty("core.error.http_status");
               var exception = e['text'];
            }
            else
            {
               var message = this.getProperty("core.error.general");
               var exception = e['text'];
            }

            var direction = this.getProperty("lang.direction");

            var content = '<html><body style="margin-top:40px; font-family:arial; background-color:#F4F8FD;"><center><table width="600px" style="font-size:11pt; color:#777; border:2px solid #E5F0FB; background-color:#ffffff; border-spacing:0px;"><tr><td align="center" style="background-color: #f0fff0; color: #FB9233; font-weight:bold; padding:5px; border-bottom:1px solid #E5F0FB;">EPUBReader</td></tr><tr><td style="padding:10px; direction:' + direction + ';">' + message + '</td></tr><tr><td style="padding:10px;">' + exception + '</td></tr></table></body></html>';

            return content;
         },

         getProperty: function(name)
         {
            var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                         .getService(Components.interfaces.nsIStringBundleService)
                         .createBundle("chrome://epubreader/locale/epubreader.properties");

            var property = bundle.GetStringFromName(name);

            return property;
         }
      }

      var output = "";

      try
      {
         if(request instanceof Ci.nsIHttpChannel)
         {
            var chan = request.QueryInterface(Ci.nsIChannel);
            var httpChannel = chan.QueryInterface(Ci.nsIHttpChannel);
            var responseStatus = httpChannel.responseStatus;
            var responseStatusText = httpChannel.responseStatusText;

            if(responseStatus == 401 || responseStatus == 403)
            {
               var exception = [];
               exception['code'] = epub.exception_http_status_not_authorized;

               throw(exception);
            }
            else if(responseStatus == 404)
            {
               var exception = [];
               exception['code'] = epub.exception_http_status_not_found;

               throw(exception);
            }
            else if(responseStatus >= 400)
            {
               var exception = [];
               exception['code'] = epub.exception_http_status_other;
               exception['text'] = "HTTP: " + responseStatus + ", " + responseStatusText;

               throw(exception);
            }
         }

         var uri = request.QueryInterface(Ci.nsIChannel).URI.spec;
         id = epub.read(this.data, uri, EPUBReader.contentDisposition);
         browser.redirect(id, EPUBReader.openFrom, EPUBReader.currentUri);
      }
      catch(e)
      {
         output = html.get(e);
      }

      var storage = Cc["@mozilla.org/storagestream;1"].createInstance(Ci.nsIStorageStream);
      storage.init(4, 0xffffffff, null);
      var out = storage.getOutputStream(0);

      var binout = Cc["@mozilla.org/binaryoutputstream;1"].createInstance(Ci.nsIBinaryOutputStream);
      binout.setOutputStream(out);
      binout.writeUtf8Z(output);
      binout.close();

      var trunc = 4;
      var instream = storage.newInputStream(trunc);

      this.listener.onDataAvailable(this.channel, context, instream, 0, storage.length - trunc);
      this.listener.onStopRequest(request, context, statusCode);
   },
   /* nsIRequestObserver end */

   getEpubList: function(tag_id)
   {
      var epubs = epub.getEpubList(tag_id);
      return epubs;
   },

   deleteEpub: function(id)
   {
      try
      {
         epub.deleteEpub(id);
         return true;
      }
      catch(e)
      {
         return false;
      }
   },

   deleteEpubPermanent: function(id)
   {
      try
      {
         epub.deleteEpubPermanent(id);
         return true;
      }
      catch(e)
      {
         return false;
      }
   },

   deleteEpubsPermanent: function()
   {
      try
      {
         epub.deleteEpubsPermanent();
         return true;
      }
      catch(e)
      {
         return false;
      }
   },

   updateCSS: function()
   {
      epub.updateCSS();
   },

   getEpub: function(id)
   {
      var myEpub = epub.getEpub(id);
      return myEpub;
   },

   getEpubs: function()
   {
      var epubs = epub.getEpubs();
      return epubs;
   },

   getDeletedEpubs: function()
   {
      var epubs = epub.getDeletedEpubs();
      return epubs;
   },

   getEpubByIdentifier: function(id)
   {
      var myEpub = epub.getEpubByIdentifier(id);
      return myEpub;
   },

   getEpubIdentifier: function(file)
   {
      var id = epub.getEpubIdentifier(file);
      return id;
   },

   getEpubRoot: function()
   {
      var root = epub.getEpubRoot();
      return root;
   },

   getMetadata: function(id)
   {
      var meta = epub.getMetadata(id);
      return meta;
   },

   storeBookmark: function(bookmark)
   {
      epub.storeBookmark(bookmark);
   },

   getBookmarks: function(id)
   {
      var bookmarks = epub.getBookmarks(id);
      return bookmarks;
   },

   getBookmarkByName: function(id, name)
   {
      var bookmark = epub.getBookmarkByName(id, name);
      return bookmark;
   },

   upgradeTasks: function(oldVersion, newVersion)
   {
      epub.upgradeTasks(oldVersion, newVersion);
   },

   readURL: function(url)
   {
      var data = epub.readURL(url);
      return data;
   },

   setCatalog: function(catalog)
   {
      epub.catalog = catalog;
   },

   getCatalog: function()
   {
      return epub.catalog;
   },

   storeCatalog: function(catalog)
   {
      epub.storeCatalog(catalog);
   },

   getCatalogs: function()
   {
      return epub.getCatalogs();
   },

   getCatalogByName: function(name)
   {
      var catalog = epub.getCatalogByName(name);

      return catalog;
   },

   storeTag: function(name)
   {
      try
      {
         var ret = epub.storeTag(name);
         return ret;
      }
      catch(e)
      {
         return false;
      }
   },

   updateTag: function(id, name)
   {
      try
      {
         var ret = epub.updateTag(id, name);
         return ret;
      }
      catch(e)
      {
         return false;
      }
   },

   deleteTag: function(id)
   {
      try
      {
         var ret = epub.deleteTag(id);
         return ret;
      }
      catch(e)
      {
         return false;
      }
   },

   getTags: function()
   {
      var tags = epub.getTags();

      return tags;
   },

   getBookTags: function(id)
   {
      var tags = epub.getBookTags(id);

      return tags;
   },

   storeBookTags: function(id, bookTags)
   {
      try
      {
         epub.storeBookTags(id, bookTags);
         return true;
      }
      catch(e)
      {
         return false;
      }
   },

   importEpub: function(file)
   {
      try
      {
         epub.importEpub(file);
         return true;
      }
      catch(e)
      {
         return e['code'];
      }
   },

   exportEpub: function(id, dir)
   {
      try
      {
         epub.exportEpub(id, dir);
         return true;
      }
      catch(e)
      {
         return e['code'];
      }
   }
};

/* register component */
var components = [EPUBReader];

if(XPCOMUtils.generateNSGetFactory)
{
   var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
}
else
{
   var NSGetModule = XPCOMUtils.generateNSGetModule(components);
}