/**
 * EPUBReader Firefox Extension: http://www.epubread.com/
 * Copyright (C) 2009 Michael Volz (epubread@gmail.com)
 *
 * This program is free software: you can redistribute it under
 * the terms of the attached license (license.txt).
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * attached license (license.txt) for more details.
 *
 * You should have received a copy of the license (license.txt)
 * along with this program. If not, see <http://www.epubread.com/license/>.
**/

var catalog =
{
   excerptUrl: null,
   items: null,
   selectedItem: null,
   selectedRow: null,
   currentPage: null,
   navPages: null,
   navPageNumber: null,
   navBackwards: null,
   navForwards: null,
   mode: null,
   catalogs: null,
   selectedCatalog: null,
   ePubReader: null,
   pref: null,
   opdsLang: null,

   menu_private: -2,

   tag_library: -1,
   tag_add: -2,

   catalog_shop: "ef57c8cbcb29d1453265e807bf91afdf",

   mode_none: 0,
   mode_navigate_backwards: 1,
   mode_navigate_forwards: 2,
   mode_navigate_reload: 3,
   mode_details: 4,

   nav_backwards: 1,
   nav_forwards: 2,

   error_timeout: -1,
   error_no_connection: -2,
   error_other: -3,

   message_error: 1,
   message_info: 2,

   init: function()
   {
      var epubReader = Components.classes["@mozilla.org/streamconv;1?from=application/epub+zip&to=*/*"]
                       .getService(Components.interfaces.nsISupports).wrappedJSObject;

      var pref = Components.classes["@mozilla.org/preferences-service;1"]
                 .getService(Components.interfaces.nsIPrefService).getBranch("extensions.epubreader.");

      catalog.pref = pref;

      catalog.epubReader = epubReader;
      catalog.epubReader.setCatalog(catalog);

      catalog.fillCatalogMenulist();
      catalog.mode = catalog.mode_none;

      var title = catalog.getProperty("catalog.title");
      document.title = title;

      var catalogTreeWidth = catalog.pref.getCharPref("width.catalog.tree");

      var catalogTree = document.getElementById("catalog_tree_box");
      catalogTree.width = catalogTreeWidth;

      var itemsFrame = document.getElementById("catalog_items_frame");
      itemsFrame.addEventListener("load", catalog.reloadItems, true);
   },

   fillCatalogMenulist: function()
   {
      const xul_ns = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

      var catalogs = this.epubReader.getCatalogs();
      catalog.catalogs = this.check(catalogs);

      var catalogList = document.getElementById("catalog_menulist");
      var library = this.getProperty("catalog.library");

      var catalogListPrivate = document.getElementById("catalog_menulist_private");
      catalogListPrivate.setAttribute("label", library);
      catalogListPrivate.setAttribute("value", catalog.menu_private);

      var count = catalogList.itemCount;

      for(var i = count - 1; i > 1; i--)
      {
         catalogList.removeItemAt(i);
      }

      if(this.getLocale().match(/(de.*)/i))
      {
         for(var i = 0; i < catalog.catalogs.length; i++)
         {
            if(catalog.catalogs[i]['code'] == this.catalog_shop)
            {
               var name = catalog.catalogs[i]['name'];
               var languageShort = catalog.catalogs[i]['language'];

               if(languageShort != "")
               {
                  var languageLong = this.getProperty("lang." + languageShort);
                  var desc = name + ", " + languageLong;
               }
               else
               {
                  var desc = name;
               }

               catalogList.appendItem(desc, i);

               var catalogListPopup = document.getElementById("catalog_menulist_popup");
               var separator = document.createElement("menuseparator");
               catalogListPopup.appendChild(separator);

               break;
            }
         }
      }

      for(var i = 0; i < catalog.catalogs.length; i++)
      {
         if(catalog.catalogs[i]['code'] != this.catalog_shop)
         {
            var name = catalog.catalogs[i]['name'];
            var languageShort = catalog.catalogs[i]['language'];

            if(languageShort != "")
            {
               var languageLong = this.getProperty("lang." + languageShort);
               var desc = name + ", " + languageLong;
            }
            else
            {
               var desc = name;
            }

            catalogList.appendItem(desc, i);
         }
      }

      this.selectCatalog(this.menu_private);
   },

   selectCatalog: function(id)
   {
      if(!id || id == this.menu_private ||
         (id >= 0 && this.catalogs[id]['name'] == "Gutenberg.org"))
      {
         var tree = document.getElementById("catalog_tree");
         var list = document.getElementById("catalog_list");
         tree.setAttribute("hidden", "true");
         list.removeAttribute("hidden");

         var tag = this.pref.getIntPref("library_last_tag");
         var pageFrame = document.getElementById("catalog_page_frame");

         if(id >= 0 && this.catalogs[id]['name'] == "Gutenberg.org")
         {
            var location = this.catalogs[id]['start'];

            this.selectedCatalog = id;
            this.resetCatalogListLibrary();

            var list = document.getElementById("catalog_list");
            var start = this.getProperty("catalog.start");
            list.appendItem(start, 0);
         }
         else
         {
            var location = "chrome://epubreader/content/library.xul?tag_id=" + tag;

            this.selectedCatalog = this.menu_private;
            this.resetCatalogListLibrary();
            this.fillCatalogListLibrary();
         }

         var searchBox = document.getElementById("catalog_search_box");
         searchBox.setAttribute("hidden", "true");

         pageFrame.contentWindow.location = location;
         pageFrame.style.display = "inline";

         var itemsFrame = document.getElementById("catalog_items_frame");
         itemsFrame.style.display = "none";

         var navFrame = document.getElementById("catalog_nav_frame");
         navFrame.style.display = "none";
      }
      else if(id >= 0)
      {
         var url = this.catalogs[id]['start'];

         if(url)
         {
            var tree = document.getElementById("catalog_tree");
            var list = document.getElementById("catalog_list");
            list.setAttribute("hidden", "true");
            tree.removeAttribute("hidden");

            var pageFrame = document.getElementById("catalog_page_frame");
            pageFrame.style.display = "none";
            pageFrame.contentWindow.location = "about:blank";

            if(this.catalogs[id]['search'])
            {
               var searchBox = document.getElementById("catalog_search_box");
               var search = this.getProperty("catalog.search");
               searchBox.removeAttribute("hidden");
               searchBox.setAttribute("emptytext", search);
            }
            else
            {
               var searchBox = document.getElementById("catalog_search_box");
               searchBox.setAttribute("hidden", "true");
            }

            var navFrame = document.getElementById("catalog_nav_frame");
            navFrame.style.display = "none";

            this.selectedCatalog = id;

            this.setWaitStatus(true);
            this.selectedItem = null;
            this.selectedRow = null;
            this.resetCatalogList();
            this.resetItemList();

            this.getContent(url);
         }
         else
         {
            this.selectedCatalog = null;
         }
      }
   },

   fillCatalogList: function(content, selectedItem)
   {
      const xul_ns = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

      var children = document.createElementNS(xul_ns, "treechildren");
      var entries = content['data'];

      for(var i = 0; i < entries.length; i++)
      {
         var item = document.createElementNS(xul_ns, "treeitem");
         item.setAttribute("container", "true");

         var row = document.createElementNS(xul_ns, "treerow");

         var cell = document.createElementNS(xul_ns, "treecell");
         cell.setAttribute("label", entries[i]['title']);
         row.appendChild(cell);

         var cell = document.createElementNS(xul_ns, "treecell");
         cell.setAttribute("label", entries[i]['details']['data']);
         row.appendChild(cell);

         var cell = document.createElementNS(xul_ns, "treecell");
         cell.setAttribute("label", entries[i]['details']['type']);
         row.appendChild(cell);

         var cell = document.createElementNS(xul_ns, "treecell");
         cell.setAttribute("label", "false");
         row.appendChild(cell);

         item.appendChild(row);
         children.appendChild(item);
      }

      if(selectedItem)
      {
         selectedItem.appendChild(children);
      }
      else
      {
         var tree = document.getElementById("catalog_tree");

         this.resetCatalogList();
         tree.appendChild(children);
      }

      this.setWaitStatus(false);
   },

   fillCatalogListLibrary: function()
   {
      var last = this.pref.getIntPref("library_last_tag");

      var list = document.getElementById("catalog_list");
      var entries = this.epubReader.getTags();

      var library = this.getProperty("catalog.library");
      var itemLibrary = list.appendItem(library, this.tag_library);

      if(last == this.tag_library)
      {
         var itemLast = itemLibrary;
      }

      if(entries.length > 0)
      {
         var item = list.appendItem("---------------", "");
      }

      for(var i = 0; i < entries.length; i++)
      {
         var item = list.appendItem(entries[i].name + " (" + entries[i].book_count + ")", entries[i].id);

         if(!itemLast && entries[i].id == last)
         {
            var itemLast = item;
         }
      }

      var item = list.appendItem("---------------", "");

      var tag = this.getProperty("catalog.tag");
      var item = list.appendItem("+ " + tag, this.tag_add);
      item.style.fontSize = "9pt";
      item.style.color = "grey";

      if(!itemLast)
      {
         var itemLast = itemLibrary;
         this.pref.setIntPref("library_last_tag", this.tag_library);
      }

      list.focus();
      list.selectItem(itemLast);
   },

   resetCatalogList: function()
   {
      var tree = document.getElementById("catalog_tree");
      var treeChildren = tree.getElementsByTagName("treechildren")[0];

      if(treeChildren)
      {
         tree.removeChild(treeChildren);
      }
   },

   resetCatalogListLibrary: function()
   {
      var list = document.getElementById("catalog_list");

      for(var i = list.getRowCount() - 1; i >= 0 ; i--)
      {
         list.removeItemAt(i);
      }
   },

   fillItemList: function(items, nav)
   {
      function get(items, shop, template)
      {
         var repeat_begin = "%repeat-begin%";
         var repeat_end = "%repeat-end%";
         var repeat = template.substring(template.search(new RegExp(repeat_begin)) + repeat_begin.length, template.search(new RegExp(repeat_end)));

         var separator_begin = "%separator-begin%";
         var separator_end = "%separator-end%";
         var separator = template.substring(template.search(new RegExp(separator_begin)) + separator_begin.length, template.search(new RegExp(separator_end)));

         var navigation_begin = "%navigation-begin%";
         var navigation_end = "%navigation-end%";
         var navigation = template.substring(template.search(new RegExp(navigation_begin)) + navigation_begin.length, template.search(new RegExp(navigation_end)));

         var excerpt = catalog.getProperty("catalog.excerpt");

         var content = "";

         for(var i = 0; i < items.length; i++)
         {
            var element = repeat;

            if(items[i]['author'])
            {
               var title = items[i]['title'] + " / " + items[i]['author'];
            }
            else
            {
               var title = items[i]['title'];
            }

            element = element.replace(/%title_author%/, makeHtmlSave(title));
            element = element.replace(/%id%/, i);

            if(items[i]['epub'])
            {
               var display = "inline";
            }
            else
            {
               var display = "none";
            }

            element = element.replace(/%epub_display%/, display);
            element = element.replace(/%epub%/, makeHtmlSave(items[i]['epub']));

            var shortened = false;
            var summary = items[i]['summary'];

            if(summary)
            {
               if(summary.length > 250)
               {
                  summary = summary.substr(0, 250);

                  var search = summary.search(/<[^>]*$/);

                  if(search != -1)
                  {
                     summary = summary.substr(0, search);
                  }

                  summary = summary + "...";
                  shortened = true;
               }
            }

            element = element.replace(/%summary%/, makeHtmlSave(summary));

            if(items[i]['thumbnail'])
            {
               element = element.replace(/%thumbnail%/, makeHtmlSave(items[i]['thumbnail']));
               var display = "inline";
            }
            else
            {
               var display = "none";
            }

            element = element.replace(/%thumbnail_display%/, display);

            if(shortened == false && !items[i]['cover'] && !items[i]['details'])
            {
               var display = "none";
            }
            else
            {
               var display = "inline";
            }

            element = element.replace(/%details_display%/, display);

            if(items[i]['buy'] && shop == true)
            {
               element = element.replace(/%buy_url%/, makeHtmlSave(items[i]['buy']));
               var display = "inline";
            }
            else
            {
               var display = "none";
            }

            element = element.replace(/%buy_display%/, display);

            if(items[i]['site'])
            {
               element = element.replace(/%site_url%/, makeHtmlSave(items[i]['site']));
               var display = "inline";
            }
            else
            {
               var display = "none";
            }

            element = element.replace(/%site_display%/, display);

            if(items[i]['excerpt'])
            {
               element = element.replace(/%excerpt_url%/, makeHtmlSave(items[i]['excerpt']));
               element = element.replace(/%excerpt_desc%/, excerpt);
               var display = "inline";
            }
            else
            {
               var display = "none";
            }

            element = element.replace(/%excerpt_display%/, display);

            content = content + element;

            if(i < items.length - 1)
            {
               content = content + separator;
            }
         }

         if(catalog.navBackwards || catalog.navForwards)
         {
            content = content + separator;

            var element = navigation;

            if(!catalog.navBackwards)
            {
               element = element.replace(/%nav_backwards_src%/, "chrome://epubreader/skin/nav_page_backwards_disabled.png");
            }
            else
            {
               element = element.replace(/%nav_backwards_src%/, "chrome://epubreader/skin/nav_page_backwards_enabled.png");
            }

            if(!catalog.navForwards)
            {
               element = element.replace(/%nav_forwards_src%/, "chrome://epubreader/skin/nav_page_forwards_disabled.png");
            }
            else
            {
               element = element.replace(/%nav_forwards_src%/, "chrome://epubreader/skin/nav_page_forwards_enabled.png");
            }

            content = content + element;
         }

         template = template.replace(/%repeat-begin%(.|\s)*%navigation-end%/, content);
         return template;
      }

      function makeHtmlSave(text)
      {
         if(text)
         {
            /*
               text = text.replace(/</g, "&lt;");
               text = text.replace(/>/g, "&gt;");
               text = text.replace(/&/g, "&amp;");
               text = text.replace(/"/g, "&quot;");
               text = text.replace(/'/g, "&apos;");
            */
         }

         return text;
      }

      this.items = items;

      var url = "chrome://epubreader/content/catalog_items_template.html";
      var template = this.epubReader.readURL(url);
      var shop = this.checkShop();

      var content = get(items, shop, template);

      var catalogFrame = document.getElementById("catalog_items_frame");
      var table = catalogFrame.contentDocument.getElementById("catalog_table");
      table.innerHTML = content;

      var pageFrame = document.getElementById("catalog_page_frame");

      pageFrame.style.display = "none";
      pageFrame.contentWindow.location = "about:blank";
      catalogFrame.style.display = "";

      this.setWaitStatus(false);
   },

   resetItemList: function()
   {
      var frame = document.getElementById("catalog_items_frame");
      var table = frame.contentDocument.getElementById("catalog_table");
      table.innerHTML = "";

      dump("reset\n");
   },

   navigate: function(direction)
   {
      var url = null;

      if(direction == this.nav_backwards && this.navBackwards)
      {
         url = this.navBackwards;
         this.mode = this.mode_navigate_backwards;
      }
      else if(direction == this.nav_forwards && this.navForwards)
      {
         url = this.navForwards;
         this.mode = this.mode_navigate_forwards;
      }

      if(url)
      {
         this.resetItemList();

         var text = this.getProperty("catalog.loading");
         this.showMessageBox(text, this.message_info, false);
         this.showWaitCursor();

         this.getContent(url);
      }
   },

   search: function(query)
   {
      var url = this.catalogs[this.selectedCatalog]['search'];

      //url = url.replace(/[<>&]/g, " ");
      query = query.replace(/ /g, "+");
      url = url.replace(/%search%/, query);

      this.resetItemList();
      this.setWaitStatus(true);

      frame = document.getElementById("catalog_page_frame");
      frame.style.display = "none"
      frame.contentWindow.location = "about:blank";

      this.getContent(url);
   },

   showDetails: function(id)
   {
      if(this.items[id]['details'])
      {
         this.setWaitStatus(true);
         this.mode = this.mode_details;
         this.getContent(this.items[id]['details']);
      }
      else
      {
         this.getDetails(this.items[id]);
      }
   },

   getDetails: function(data)
   {
      function get(details, shop, template)
      {
         var repeat_begin = "%repeat-begin%";
         var repeat_end = "%repeat-end%";
         var repeat = template.substring(template.search(new RegExp(repeat_begin)) + repeat_begin.length, template.search(new RegExp(repeat_end)));

         var content = "";

         var element = repeat;

         if(details['author'])
         {
            var title = details['title'] + " / " + details['author'];
         }
         else
         {
            var title = details['title'];
         }

         element = element.replace(/%title_author%/, makeHtmlSave(title));
         element = element.replace(/%summary%/, makeHtmlSave(details['summary']));

         if(details['epub'])
         {
            display = "inline";
         }
         else
         {
            display = "none";
         }

         element = element.replace(/%epub_display%/, display);
         element = element.replace(/%epub%/, makeHtmlSave(details['epub']));

         if(details['cover'])
         {
            var display = "inline";
         }
         else
         {
            var display = "none";
         }

         element = element.replace(/%cover_display%/, display);
         element = element.replace(/%cover%/, makeHtmlSave(details['cover']));

         if(details['buy'] && shop == true)
         {
            display = "inline";
         }
         else
         {
            display = "none";
         }

         element = element.replace(/%buy_display%/, display);
         element = element.replace(/%buy_url%/, makeHtmlSave(details['buy']));

         if(details['excerpt'])
         {
            display = "inline";
         }
         else
         {
            display = "none";
         }

         var excerpt = catalog.getProperty("catalog.excerpt");

         element = element.replace(/%excerpt_display%/, display);
         element = element.replace(/%excerpt_url%/, makeHtmlSave(details['excerpt']));
         element = element.replace(/%excerpt_desc%/, excerpt);

         content = content + element;

         template = template.replace(/%repeat-begin%(.|\s)*%repeat-end%/, content);
         return template;
      }

      function makeHtmlSave(text)
      {
         if(text)
         {
            /*
               text = text.replace(/</g, "&lt;");
               text = text.replace(/>/g, "&gt;");
               text = text.replace(/&/g, "&amp;");
               text = text.replace(/"/g, "&quot;");
               text = text.replace(/'/g, "&apos;");
            */
         }

         return text;
      }

      var url = "chrome://epubreader/content/catalog_item_details_template.html";
      var template = this.epubReader.readURL(url);
      var shop = this.checkShop();

      var content = get(data, shop, template);

      var frame = document.getElementById("catalog_item_details_frame");
      var table = frame.contentDocument.getElementById("catalog_item_details");
      table.innerHTML = content;

      frame.style.paddingLeft = "10px";
      frame.style.paddingTop = "5px";
      frame.style.display = "";

      this.setWaitStatus(false);
   },

   closeDetails: function()
   {
      var frame = document.getElementById("catalog_item_details_frame");
      frame.style.display = "none";
   },

   handleContent: function(content)
   {
      if(!content || !this.checkCalibre(content['id']))
      {
         if(this.selectedItem)
         {
            var tree = document.getElementById("catalog_tree");
            var loadedColumn = tree.treeBoxObject.columns.getNamedColumn("catalog_tree_col_loaded");

            tree.view.setCellText(this.selectedRow, loadedColumn, "true");
            this.selectedItem.setAttribute("container", "false");
         }

         this.showDefaultCursor();
         var text = this.getProperty("catalog.nodata");
         this.showMessageBox(text, this.message_info, true);
      }
      else
      {
         if(this.selectedItem)
         {
            var tree = document.getElementById("catalog_tree");

            try
            {
               var loadedColumn = tree.treeBoxObject.columns.getNamedColumn("catalog_tree_col_loaded");
               tree.view.setCellText(this.selectedRow, loadedColumn, "true");
            }
            catch(e)
            {
            }
         }

         this.opdsLang = content['lang'];

         if(content['type'] == "categories")
         {
            this.fillCatalogList(content, this.selectedItem);
         }
         else
         {
            if(this.mode == this.mode_details)
            {
               this.getDetails(content['data'][0]);
            }
            else
            {
               if(this.mode == this.mode_navigate_backwards)
               {
                  this.navPageNumber--;

                  if(this.navPageNumber == 0)
                  {
                     this.navBackwards = null;
                  }
                  else
                  {
                     this.navBackwards = this.navPages[this.navPageNumber - 1];
                  }
               }
               else if(this.mode == this.mode_navigate_forwards)
               {
                  this.navPageNumber++;
                  this.navBackwards = this.navPages[this.navPageNumber - 1];
               }
               else if(this.mode == this.mode_navigate_reload)
               {
                  if(this.navPageNumber > 0)
                  {
                     this.navBackwards = this.navPages[this.navPageNumber - 1];
                  }
               }

               this.navPages[this.navPageNumber] = this.currentPage;

               if(content['nav'] && content['nav']['forwards'])
               {
                  this.navForwards = content['nav']['forwards'];
               }
               else
               {
                  this.navForwards = null;
               }

               if(this.selectedItem)
               {
                  this.selectedItem.setAttribute("container", "false");
               }

               this.fillItemList(content['data'], content['nav']);
            }
         }
      }

      this.mode = this.mode_none;
      this.selectedItem = null;
      this.selectedRow = null;

      var searchBox = document.getElementById("catalog_search_box");
      searchBox.value = "";
   },

   handleSelection: function(row, items, type)
   {
      var tree = document.getElementById("catalog_tree");
      var item = tree.view.getItemAtIndex(row);
      var url = items;

      this.resetItemList();
      this.setWaitStatus(true);

      if(type == "items")
      {
         this.selectedItem = item;
         this.selectedRow = row;

         frame = document.getElementById("catalog_page_frame");
         frame.style.display = "none"
         frame.contentWindow.location = "about:blank";

         this.getContent(url);
      }
      else if(type == "page")
      {
         item.setAttribute("container", "false");
         this.loadPage(url);
      }
   },

   treeSelected: function(event)
   {
      var tree = document.getElementById("catalog_tree").treeBoxObject;
      var valueColumn = tree.columns.getNamedColumn("catalog_tree_col_value");
      var typeColumn = tree.columns.getNamedColumn("catalog_tree_col_type");
      var loadedColumn = tree.columns.getNamedColumn("catalog_tree_col_loaded");

      var row = tree.getRowAt(event.clientX, event.clientY);

      var loaded = tree.view.getCellText(row, loadedColumn);
      var type = tree.view.getCellText(row, typeColumn);
      var item = tree.view.getItemAtIndex(row);

      if((loaded != "true" ||
          item.getAttribute("container") == "false") ||
         (type == "page"))
      {
         var items = tree.view.getCellText(row, valueColumn);
         this.handleSelection(row, items, type);
      }
   },

   listSelected: function()
   {
      if(this.selectedCatalog == this.menu_private)
      {
         var list = document.getElementById("catalog_list");
         var item = list.getItemAtIndex(list.currentIndex);
         var id = item.value;

         if(id == this.tag_add)
         {
            this.pref.setIntPref("library_last_tag", this.tag_library);
            this.openTagDialog(false);
         }
         else if(id == this.tag_library || id > 0)
         {
            this.pref.setIntPref("library_last_tag", id);
            var pageFrame = document.getElementById("catalog_page_frame");
            pageFrame.contentWindow.location = "chrome://epubreader/content/library.xul?tag_id=" + id;
         }
      }
      else
      {
         var pageFrame = document.getElementById("catalog_page_frame");
         pageFrame.contentWindow.location = this.catalogs[this.selectedCatalog]['start'];
      }
   },

   getContent: function(url)
   {
      if(this.mode == this.mode_none)
      {
         this.navPageNumber = 0;
         this.navPages = [];
         this.navBackwards = null;
         this.navForwards = null;
      }

      this.currentPage = url;
      opdsreader.getContent(url);
   },

   handleError: function(type)
   {
      this.showDefaultCursor();
      this.mode = this.mode_none;

      var searchBox = document.getElementById("catalog_search_box");
      searchBox.value = "";

      if(type == this.error_timeout)
      {
         var text = this.getProperty("catalog.error.timeout");
      }
      else if(type == this.error_no_connection)
      {
         var text = this.getProperty("catalog.error.no_connection");
      }
      else
      {
         var text = this.getProperty("catalog.error.other");
      }

      this.showMessageBox(text, this.message_error, true);
   },

   setWaitStatus: function(set)
   {
      if(set == true)
      {
         this.showWaitCursor();
         var text = this.getProperty("catalog.loading");
         this.showMessageBox(text, this.message_info, false);
      }
      else
      {
         this.showDefaultCursor();
         this.closeMessageBox();
      }
   },

   showMessageBox: function(text, type, autoclose)
   {
      var box = document.getElementById("catalog_message_box");
      var table = document.getElementById("catalog_message_table");
      var message = document.getElementById("catalog_message_box_message");

      if(message.hasChildNodes())
      {
         var textNode = message.firstChild;
      }
      else
      {
         var textNode = document.createTextNode("");
         message.appendChild(textNode);
      }

      textNode.nodeValue = text;

      if(type == this.message_info)
      {
         table.setAttribute("class", "info");
      }
      else if(type == this.message_error)
      {
         table.setAttribute("class", "error");
      }

      var treeBox = document.getElementById("catalog_tree_box");
      var stack = document.getElementById("catalog_items_stack");
      var top = 70;
      var left = treeBox.clientWidth + stack.clientWidth/2 - 125;

      box.top = top;
      box.left = left;
      box.style.display = "";

      if(autoclose == true)
      {
         window.setTimeout(this.closeMessageBox, 3000);
      }
   },

   closeMessageBox: function()
   {
      var box = document.getElementById("catalog_message_box");
      box.style.display = "none";
   },

   showWaitCursor: function()
   {
      var catalog = document.getElementById("catalog");
      var items = document.getElementById("catalog_items_frame").contentDocument;
      var itemsHtml = items.getElementsByTagName("html")[0];
      var itemsTable = items.getElementById("catalog_table");

      catalog.style.cursor = "wait";
      itemsHtml.style.cursor = "wait";
      itemsTable.style.cursor = "wait";
   },

   showDefaultCursor: function()
   {
      var catalog = document.getElementById("catalog");
      var items = document.getElementById("catalog_items_frame").contentDocument;
      var itemsHtml = items.getElementsByTagName("html")[0];
      var itemsTable = items.getElementById("catalog_table");

      catalog.style.cursor = "default";
      itemsHtml.style.cursor = "default";
      itemsTable.style.cursor = "default";
   },

   loadPage: function(url)
   {
      //dump("loadPage: " + url + "\n");
      this.showDefaultCursor();
      this.closeMessageBox();

      var pageFrame = document.getElementById("catalog_page_frame");
      pageFrame.contentWindow.location = url;
      pageFrame.style.display = "inline";
   },

   showBuy: function(url)
   {
      //dump(url + "\n");
      var pageFrame = document.getElementById("catalog_page_frame");

      pageFrame.style.display = "inline";
      pageFrame.contentWindow.location = url;

      if(this.checkShop())
      {
         var navFrame = document.getElementById("catalog_nav_frame");

         var shopping = navFrame.contentDocument.getElementById("shopping");
         var shoppingText = this.getProperty("catalog.shopping");
         shopping.innerHTML = shoppingText;

         var basket = navFrame.contentDocument.getElementById("basket");
         var basketText = this.getProperty("catalog.basket");
         basket.innerHTML = basketText;

         navFrame.style.display = "inline";
      }
   },

   showBasket: function()
   {
      var pageFrame = document.getElementById("catalog_page_frame");

      var url = this.catalogs[this.selectedCatalog]['start'];
      var baseUrl = "https" + url.substring(url.indexOf(":"), url.lastIndexOf("/") + 1);

      if(this.opdsLang == "en")
      {
         baseUrl = baseUrl + "en/";
      }

      pageFrame.contentWindow.location = baseUrl + "order.php";
      pageFrame.style.display = "inline";
   },

   showExcerpt: function(url)
   {
      //dump(url + "\n");
      this.excerptUrl = url;

      var pageFrame = document.getElementById("catalog_page_frame");
      pageFrame.addEventListener("load", catalog.loadExcerpt, true);

      pageFrame.style.display = "inline";
      pageFrame.contentWindow.location = "chrome://epubreader/content/excerpt.html";
   },

   loadExcerpt: function()
   {
      var pageFrame = document.getElementById("catalog_page_frame");

      pageFrame.removeEventListener("load", catalog.loadExcerpt, true);

      var excerptFrame = pageFrame.contentDocument.getElementById("excerpt_frame");
      excerptFrame.addEventListener("load", catalog.modifyExcerpt, true);
      excerptFrame.contentWindow.location = catalog.excerptUrl;

      catalog.excerptUrl = null;
   },

   modifyExcerpt: function()
   {
      var pageFrame = document.getElementById("catalog_page_frame");
      var excerptFrame = pageFrame.contentDocument.getElementById("excerpt_frame");
      excerptFrame.removeEventListener("load", catalog.modifyExcerpt, true);

      var doc = excerptFrame.contentDocument;
      doc.body.style.backgroundColor = "#ffffff";
      doc.body.style.fontFamily = "arial";
      doc.body.style.fontSize = "11pt";
      doc.body.style.color = "#777";
      doc.body.style.paddingLeft = "10px";
      doc.body.style.paddingRight = "10px";
   },

   close: function()
   {
      var pageFrame = document.getElementById("catalog_page_frame");
      pageFrame.style.display = "none";
      pageFrame.contentWindow.location = "about:blank";
   },

   openTagDialog: function(update)
   {
      var list = document.getElementById("catalog_list");
      var item = list.getItemAtIndex(list.currentIndex);

      window.openDialog("chrome://epubreader/content/library_tags.xul", "", "modal=yes,left=100,top=200", update, item.value, item.label);
      window.location.reload();
   },

   modifyTagMenu: function()
   {
      var list = document.getElementById("catalog_list");
      var item = list.getItemAtIndex(list.currentIndex);
      var id = item.value;

      var popup = document.getElementById("tag_popup");

      if(id > 0)
      {
         document.getElementById("tag_popup_change").hidden = false;
         document.getElementById("tag_popup_delete").hidden = false;
      }
      else
      {
         document.getElementById("tag_popup_change").hidden = true;
         document.getElementById("tag_popup_delete").hidden = true;
      }
   },

   deleteTag: function()
   {
      var list = document.getElementById("catalog_list");
      var item = list.getItemAtIndex(list.currentIndex);
      var id = item.value;

      if(id > 0)
      {
         var last = this.pref.getIntPref("library_last_tag");

         if(last == id)
         {
            this.pref.setIntPref("library_last_tag", this.tag_library);
         }

         this.epubReader.deleteTag(id);
         window.location.reload();
      }
   },

   getProperty: function(name)
   {
      var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                   .getService(Components.interfaces.nsIStringBundleService)
                   .createBundle("chrome://epubreader/locale/epubreader.properties");

      var property = bundle.GetStringFromName(name);

      return property;
   },

   checkCalibre: function(id)
   {
      var catalogMenulist = document.getElementById("catalog_menulist");

      if(catalogMenulist.selectedItem.label == "Calibre2Opds")
      {
         if(!id || !id.match(/calibre.*/i))
         {
            return false;
         }
         else
         {
            return true;
         }
      }
      else
      {
         return true;
      }
   },

   checkShop: function()
   {
      var code = this.catalogs[this.selectedCatalog]['code'];

      if(code == this.catalog_shop)
      {
         var shop = true;
      }
      else
      {
         var shop = false;
      }

      return shop;
   },

   saveTreeWidth: function()
   {
      var pref = Components.classes["@mozilla.org/preferences-service;1"]
                 .getService(Components.interfaces.nsIPrefService).getBranch("extensions.epubreader.");

      var treeWidth = document.getElementById("catalog_tree_box").width;

      if(treeWidth > 0)
      {
         pref.setCharPref("width.catalog.tree", treeWidth + "px");
      }
   },

   getLocale: function()
   {
      try
      {
         var chromeRegService = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService();
         var xulChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIXULChromeRegistry);
         var toolkitChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIToolkitChromeRegistry);
         var locale = toolkitChromeReg.getSelectedLocale("epubreader");
      }
      catch(e)
      {
         var locale = "";
      }

      return locale;
   },

   reloadItems: function()
   {
      dump("reload\n");
      catalog.mode = catalog.mode_navigate_reload;
      catalog.getContent(catalog.currentPage);
   },

   showSite: function(url)
   {
      const Cc = Components.classes;
      const Ci = Components.interfaces;

      var found = false;
      var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
      var browserEnumerator = wm.getEnumerator("navigator:browser");

      while(!found && browserEnumerator.hasMoreElements())
      {
         var browserWin = browserEnumerator.getNext();
         var tabbrowser = browserWin.gBrowser;

         var numTabs = tabbrowser.browsers.length;

         for(var index = 0; index < numTabs; index++)
         {
            var currentBrowser = tabbrowser.getBrowserAtIndex(index);

            if(url == unescape(currentBrowser.currentURI.spec))
            {
               tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];
               browserWin.focus();

               found = true;
               break;
            }
         }
      }

      if(!found)
      {
         var recentWindow = wm.getMostRecentWindow("navigator:browser");

         if(recentWindow)
         {
            if("delayedOpenTab" in recentWindow)
            {
               recentWindow.delayedOpenTab(url);
            }
            else if("openNewTabWith" in recentWindow)
            {
               recentWindow.openNewTabWith(url);
            }
         }
         else
         {
            window.open(url);
         }
      }
   },

   check: function(catalogs)
   {
      function toHexString(charCode)
      {
         return ("0" + charCode.toString(16)).slice(-2);
      }

      var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                      .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);

      var ch = Components.classes["@mozilla.org/security/hash;1"]
               .createInstance(Components.interfaces.nsICryptoHash);

      var catalogsOut = [];
      var catalogsCounter = 0;

      for(var i = 0; i < catalogs.length; i++)
      {
         var string = catalogs[i]['start'] + catalogs[i]['search'] + "6sx4Al";

         converter.charset = "UTF-8";
         var result = {};
         var data = converter.convertToByteArray(string, result);

         ch.init(ch.MD5);
         ch.update(data, data.length);
         var hash = ch.finish(false);
         var hashString = [toHexString(hash.charCodeAt(j)) for (j in hash)].join("");

         if(hashString == catalogs[i]['code'] || catalogs[i]['name'] == "Calibre2Opds")
         {
            catalogsOut[catalogsCounter] = catalogs[i];
            catalogsCounter++;
         }
      }

      return catalogsOut;
   }
}

window.addEventListener("load", catalog.init, true);